import select, os, time
from gadgets.errors import GadgetsError

class RisingOrFalling(object):

    def __eq__(self, other):
        return other == '0\n' or other == '1\n'


class Poller(object):
    """
    Poller performs a poll on a gpio line.  Just call wait and it will block
    until the gpio pin goes high.

        >>> from gadgets.pins.beaglebone import pins
        >>> from gadgets.io import GPIO
        >>> poller = Poller(pins['gpio'][8][3])
        >>> poller.wait()

    This will then block until the pin goes high.
    """
    
    _export_path = '/sys/class/gpio/export'
    _mux_path = '/sys/kernel/debug/omap_mux/{0}'
    _base_path = '/sys/class/gpio/gpio{0}/{1}'
    _gpio_path = '/sys/class/gpio/gpio{0}'

    def __init__(self, pin, timeout=None, edge='rising'):
        if edge == 'rising':
            self._value = '1\n'
        elif edge == 'falling':
            self._value = '0\n'
        else:
            self._value = RisingOrFalling()
        self._edge = edge
        self._setup_pin(pin)
        if timeout:
            self._timeout = int(timeout * 1000.0)
        else:
            self._timeout = -1

    def _setup_pin(self, pin):
        mux = pin['mux']
        export = pin['export']
        self._path = self._base_path.format(export, 'value')
        self._write_to_path('27', self._mux_path.format(mux))
        self._write_to_path(str(export), self._export_path)
        self._write_to_path('in', self._base_path.format(export, 'direction'))
        self._write_to_path(self._edge, self._base_path.format(export, 'edge'))
        self._fd = None
        self._poller = None
        path = self._gpio_path.format(export)
        if not os.path.exists(path):
            raise GadgetsError('failed gpio export: {0}, mux: {1}, export: {2}'.format(path, mux, export))

    @property
    def fd(self):
        if self._fd is None:
            self._fd = os.open(self._path, os.O_RDONLY | os.O_NONBLOCK)
        return self._fd

    @property
    def poller(self):
        if self._poller is None:
            self._poller = self._get_poller()
        return self._poller

    def close(self):
        """
        Closes the file descriptor that Poller uses for the
        Linux poll.
        """
        os.close(self.fd)

    def wait(self):
        """
        Blocks until the pin goes high
        """
        while True:
            events = self.poller.poll(self._timeout)
            os.lseek(self.fd, 0, 0)
            val = os.read(self.fd, 2)
            if val == self._value:
                return events, val
        
    def _get_poller(self):
        os.read(self.fd, 2)
        poller = select.poll()
        poller.register(self.fd, select.POLLPRI)
        return poller

    def _write_to_path(self, value, path):
        try:
            f = open(path, 'w')
            f.write(str(value))
            f.close()
        except IOError:
            pass



import uuid
from gadgets.devices.trigger import Trigger

class TemperatureTriggerFactory(object):

    def __init__(self, location, addresses, comparitor=None):
        self._location = location
        self._addresses = addresses
        if comparitor is None:
            self._comparitor = lambda x, y: x >= y
        else:
            self._comparitor = comparitor
        
    def __call__(self, on_event, off_event, message, target=None):
        return TemperatureTrigger(self._location, on_event, off_event, message, self._addresses, self._comparitor, target=target)
        

class TemperatureTrigger(Trigger):

    _units = ['celcius', 'C', 'fahrenheit', 'F']

    def __init__(self, location, on_event, off_event, message, addresses, comparitor, target=None):
        super(TemperatureTrigger, self).__init__(location, on_event, off_event, message, addresses)
        self._events = ['update temperature', 'shutdown']
        self._target = target
        self._target_temperature = self._get_temperature(self._message)
        self._comparitor = comparitor

    def _get_temperature(self, message):
        if message.get('units') in self._units:
            units = message['units']
            value = message['value']
            if units == 'F' or units == 'fahrenheit':
                value = (value * 1.8) + 32.0
            return value

    def _is_mine(self, message):
        return message.get(self._location, {}).get('temperature') is not None

    def _target_reached(self, message):
        if not self._is_mine(message):
            ret = False
        else:
            temperature = self._get_temperature(message[self._location]['temperature'])
            print temperature
            ret = self._comparitor(temperature, self._target_temperature)
            print ret
        return ret
        
    def wait(self):
        event, message = self.sockets.recv()
        if event != 'shutdown' and not self._target_reached(message):
            self.wait()



from gadgets.devices.heater.pwm import PWMFactory
from gadgets.devices.heater.electric_heater import ElectricHeater

def electric_heater_factory(location, name, pin, addresses, io_factory=None, on=None, off=None):
    if io_factory is None:
        io_factory = PWMFactory(pin)
    return ElectricHeater(
        location,
        name,
        addresses,
        io_factory=io_factory,
        on=arguments.get('on'),
        off=arguments.get('off')
    )
    



from gadgets.devices.switch.switch import Switch
from gadgets.devices.switch.shift_register.io import ShiftRegisterIOFactory
from gadgets.devices.switch.shift_register.server import ShiftRegisterServer


def shift_register_switch_factory(location, name, arguments, addresses, io_factory=None):
    if shift_register_switch_factory.server is None:
        """
        The first time a ShiftRegisterSwitch gets created a ShiftRegisterServer
        must be created and started.
        """
        shift_register_switch_factory.server = ShiftRegisterServer('', 'shift register server', addresses)
        shift_register_switch_factory.server.start()
    channel = arguments['channel']
    shift_register_switch_factory.server.register_switch(channel)
    return Switch(
        location=location,
        name=name,
        addresses=addresses,
        io_factory=ShiftRegisterIOFactory(addresses, channel),
        on=arguments.get('on'),
        off=arguments.get('off'),
        momentary=arguments.get('momentary', False)
    )

shift_register_switch_factory.server = None



import time
from gadgets.devices.valve.triggers import FloatTrigger
from gadgets import Addresses

class FakePoller(object):

    def __init__(self, *args, **kw):
        pass

    def wait(self):
        time.sleep(0.4)

    def close(self):
        pass

class TestFloatTrigger(object):

    def setup(self):
        self.addresses = Addresses()
        FloatTrigger._poller_class = FakePoller
        self.trigger = FloatTrigger(
            'tank',
            'fill tank',
            'stop filling tank',
            {'units': 'liters', 'value': 3.3},
            self.addresses,
            self.off,
            {'mux':'x', 'export': 'y'},
            20.0,
        )
        self._off = False

    def off(self, message):
        self._off = True

    def test_create(self):
        pass

    def test_run(self):
        self.trigger.start()
        while not self._off:
            pass
        assert self._off



import time
import threading

from gadgets import Gadget
from gadgets.io import Poller

class ButtonPollerFactory(object):

    def __init__(self, location, name, addresses, pin):
        self._location = location
        self._name = name
        self._addresses = addresses
        self._pin = pin

    def __call__(self):
        return ButtonPoller(self._location, self._name, self._addresses, self._pin)

        
class ButtonPoller(Gadget):

    PollerClass = Poller

    def __init__(self, location, name, addresses, pin):
        self._poller = None
        self._pin = pin
        self._stop_event = threading.Event()
        super(ButtonPoller, self).__init__(location, name, addresses)
    
    @property
    def events(self):
        return []

    @property
    def poller(self):
        if self._poller is None:
            self._poller = self.PollerClass(self._pin, timeout=1, edge='both')
        return self._poller

    def close(self):
        self._stop_event.set()
        time.sleep(1)
        self.sockets.close()
        self.poller.close()
        
    def run(self):
        while not self._stop_event.is_set():
            events, val = self.poller.wait()
            if events:
                self.sockets.send('update', {self._location: {self._name: {'value': self._get_value(val)}}})

    def _get_value(self, val):
        return val == '1\n' and 'Closed' or 'Open'




import sys
import curses
import threading
import os
import time
import zmq
import imp
import collections
import json
import argparse

from gadgets.ui.data_getter import DataGetter
from gadgets.ui.database import SQL, Mongo
from gadgets.ui.container import Container
from gadgets.ui.dialogs.prompt import Prompt
from gadgets.ui.dialogs.command import Command
from gadgets.ui.dialogs.command_mode import CommandMode
from gadgets.ui.dialogs.alert import Alert
from gadgets.ui.dialogs.notes import Notes
from gadgets.ui.dialogs.method import Method
from gadgets.ui.dialogs.help import Help
from gadgets.ui.window import Window
from gadgets.ui.method_display import MethodDisplay
from gadgets import Addresses, Sockets


class UI(Window):

    Dialog = collections.namedtuple('Dialog', 'callable keyword_factory')

    def __init__(self, addresses,  db_host=None, db_port=27017, locations=[], mini=False):
        self._mini = mini
        self._addresses = addresses
        self._screen = None
        self._data = dict()
        self._getter = DataGetter(
            self.set_data,
            self._addresses
        )
        self.sockets = Sockets(addresses)
        self._lock = threading.RLock()
        self._title_bar_string = 'help: h  quit: q'
        self._location_names = locations
        self._stop = False
        self._command_mode = CommandMode(self)
        self._confirm = None
        self._commands = None
        if db_host is None:
            self._db = SQL()
        else:
            self._db = Mongo(db_host, db_port)
        self._dialogs = {
            ord('c'): self.Dialog(self._enter_command_mode, None),
            ord('C'): self.Dialog(Command(self, self._lock), None),
            ord('n'): self.Dialog(Notes(self.sockets, self._lock, self._db), self._get_method_id),
            ord('m'): self.Dialog(Method(self.sockets, self._lock, self._db), None),
            ord('f'): self.Dialog(self._user_confirmed, None),
        }
        self._dialogs[ord('h')] = self.Dialog(Help(self._lock, self._dialogs, self._db), None)

    def add_dialog(self, key, DialogClass, dialog_kw):
        self._dialogs[ord(key)] = self.Dialog(DialogClass(self.sockets, self._lock, self._db), dialog_kw)

    def _get_method_id(self):
        return {'method_id': self._data.get('method', {}).get('_id')}

    def set_data(self, data):
        self._data = data
        self._draw()
        return True

    def _do_dialog(self, x):
        dialog = self._dialogs.get(x)
        if dialog is not None:
            kw_factory = dialog.keyword_factory
            if kw_factory is not None:
                kw = kw_factory()
            else:
                kw = {}
            dialog.callable(self._screen, self._data, **kw)

    def __call__(self, screen):
        self._lock.acquire()
        self._screen = screen
        if self._mini:
            self._height = 22
        else:
            self._height = 30
        if self._mini:
            self._width = 30
        else:
            self._width = 80
        self._getter.start()
        curses.use_default_colors()
        curses.init_pair(1, curses.COLOR_YELLOW, -1)
        curses.init_pair(2, curses.COLOR_GREEN, -1)
        curses.init_pair(3, curses.COLOR_RED, -1)
        while 'locations' not in self._data:
            time.sleep(0.1)
        if self._location_names == []:
            self._location_names = self._getter.locations
        self._row, self._col = self._get_center(screen)
        self._lock.release()
        self._draw()
        x = 'go'
        
        while x != ord('q'):
            x = self._screen.getch()
            if self._command_mode.is_active:
                self._do_command_mode(x)
            else:
                self._do_dialog(x)
            self._draw()
        self._stop = True
        self._getter.stop()

    def _enter_command_mode(self, screen, data):
        """enter command mode"""
        self._command_mode.is_active = True

    def _do_command_mode(self, x):
        self._command_mode(x)

    def _user_confirmed(self, screen, data):
        if self._confirm is not None:
            self.sockets.send('completed {0}'.format(self._confirm))
            self._confirm = None

    def _draw(self):
        if self._stop:
            return
        self._lock.acquire()
        self._screen.clear()
        self._draw_top_bar()
        self._locations = self._draw_locations()
        if not self._mini:
            self._draw_method()
        self._draw_bottom_bar()
        self._screen.refresh()
        curses.curs_set(0)
        self._lock.release()

    def _draw_method(self):
        self._method_display = MethodDisplay(self._screen, self._col + 30, self._row + 2, self._data)
        self._method_display.draw()

    def _draw_locations(self):
        locations = []
        row = 1
        col = self._col
        win = self._screen.subwin(self._height, self._width, self._row + 2, self._col)
        win.border()
        for name in self._location_names:
            c =  Container(
                win,
                row,
                2,
                self._data['locations'][name],
                name,
                self._command_mode
            )
            row = c.draw()
            locations.append(c)
        return locations

    def _draw_top_bar(self):
        msg ='{0:<{1}}'.format(self._title_bar_string, self._width)
        color = curses.A_BOLD | curses.A_STANDOUT | curses.color_pair(1)
        self._screen.addstr(self._row + 1, self._col, msg, color)
        if self._command_mode.is_active:
            msg = 'COMMAND MODE'
            self._screen.addstr(self._row + 1, self._col + ((self._width / 2 - (len(msg)/2))), msg, color)

    def _draw_bottom_bar(self):
        if len(self._data.get('errors', [])) > 0:
            msg = str(self._data['errors'])
        elif self._data.get('method', {}).get('countdown') is not None:
            msg = self._data['method']['countdown']
        elif self._data.get('user_confirm'):
            self._confirm = self._data['user_confirm']
            msg = "press 'f' when {0}".format(self._confirm)
        else:
            msg = ''
        self._screen.addstr(self._row + self._height + 2, self._col, '{0}'.format(msg).center(self._width), curses.A_BOLD | curses.A_STANDOUT | curses.color_pair(1))

def get_args():
    parser = argparse.ArgumentParser(description='Start up the gadgets ui')
    parser.add_argument('--host', type=str, default='localhost')
    parser.add_argument('--port', type=int, default=5000)
    parser.add_argument('--mini', action='store_true')
    return parser.parse_args()

def main():
    args = get_args()
    config_path = os.path.join(os.path.expanduser('~'), '.gadgets', '{0}.py'.format(args.host))
    if os.path.exists(config_path):
       config = imp.load_source(args.host, config_path)
       locations = config.locations
    else:
        locations = []
    addresses = Addresses(host=args.host)
    ui = UI(addresses, locations=locations, mini=args.mini)
    curses.wrapper(ui)
    ui._getter.stop()
    os.system('stty sane')

if __name__ == '__main__':
    main()




import time, random, threading, os, uuid
from nose.tools import eq_
from gadgets.devices.valve.valve_factory import ValveFactory
from gadgets import Addresses, Gadgets, Sockets

lock = threading.Lock()
path = '/private/tmp/switch'

class FakeGPIO(object):

    def __init__(self):
        lock.acquire()
        self.status = False
        self.write('off')

    def write(self, status):
        with open(path, 'w') as f:
            f.write(status)

    def on(self):
        self.write('on')
        self.status = True

    def off(self):
        self.write('off')
        self.status = False
        if lock.locked():
            lock.release()

    def close(self):
        pass


class FakeGPIOFactory(object):

    def __init__(self, *args, **kw):
        pass
    def __call__(self, *args, **kw):
        return FakeGPIO()

class TestValve(object):

    def setup(self):
        self._off = False
        port = random.randint(5000, 50000)
        self.addresses = Addresses(in_port=port, out_port=port + 1)
        self.uid = str(uuid.uuid1())
        self.sockets = Sockets(self.addresses, events=[self.uid])

    def _get_gadgets(self, trigger_args):
        ValveFactory.gpio_factory = FakeGPIOFactory
        args = {
            'pin': None,
            'trigger': trigger_args
        }
        
        factory = ValveFactory()
        self.valve = factory(
            'bucket',
            'valve',
            args,
            self.addresses
        )
        self.gadgets = Gadgets([self.valve], self.addresses)

    def teardown(self):
        self.sockets.close()

    def is_on(self):
        with open(path, 'r') as f:
            status = f.read()
            return status == 'on'

    def test_create(self):
        trigger_args = {
            'type': 'gravity',
            'source': 'barrel',
            'tank_radius': 20.0,
            'valve_radius': 1.0,
            'valve_coefficient': 1.0
        }
        self._get_gadgets(trigger_args)
        
    def test_gravity(self):
        trigger_args = {
            'type': 'gravity',
            'source': 'barrel',
            'tank_radius': 20.0,
            'valve_radius': 1.0,
            'valve_coefficient': 1.0
        }
        self._get_gadgets(trigger_args)
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send(
            'update',
            {
                'bucket': {'volume': {'value': 1, 'units': 'liters'}},
                'barrel': {'volume': {'value': 3, 'units': 'liters'}}
            }
        )
        time.sleep(0.3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['valve']['value'], False)
        eq_(self.gadgets.coordinator._state['locations']['barrel']['volume']['value'], 3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['volume']['value'], 1)
        self.sockets.send('fill bucket', {'units': 'liters', 'value': 2})
        time.sleep(0.2)
        lock.acquire()
        lock.release()
        time.sleep(1)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['valve']['value'], False)
        self.sockets.send('shutdown')
        eq_(self.gadgets.coordinator._state['locations']['barrel']['volume']['value'], 2)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['volume']['value'], 2)

    def test_gravity_when_target_already_reached(self):
        trigger_args = {
            'type': 'gravity',
            'source': 'barrel',
            'tank_radius': 20.0,
            'valve_radius': 1.0,
            'valve_coefficient': 1.0
        }
        self._get_gadgets(trigger_args)
        
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send(
            'update',
            {
                'bucket': {'volume': {'value': 1, 'units': 'liters'}},
                'barrel': {'volume': {'value': 3, 'units': 'liters'}}
            }
        )
        time.sleep(0.3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['valve']['value'], False)
        eq_(self.gadgets.coordinator._state['locations']['barrel']['volume']['value'], 3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['volume']['value'], 1)
        self.sockets.send('fill bucket', {'units': 'liters', 'value': 1})
        eq_(self.gadgets.coordinator._state['locations']['bucket']['valve']['value'], False)
        time.sleep(0.3)
        self.sockets.send('shutdown')
        eq_(self.gadgets.coordinator._state['locations']['barrel']['volume']['value'], 3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['volume']['value'], 1)

    def test_timer_trigger_valve(self):
        f = open(path, 'w')
        f.write('')
        f.close()
        trigger_args = {
            'type': 'timer',
            'source': 'barrel',
            'drain_time': 0.5 / 60.0
        }
        self._get_gadgets(trigger_args)

        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send(
            'update',
            {
                'bucket': {'volume': {'value': 1, 'units': 'liters'}},
                'barrel': {'volume': {'value': 3, 'units': 'liters'}}
            }
        )
        time.sleep(1)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['valve']['value'], False)
        eq_(self.gadgets.coordinator._state['locations']['barrel']['volume']['value'], 3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['volume']['value'], 1)
        self.sockets.send('fill bucket', {'units': 'liters', 'value': 1})
        time.sleep(0.2)
        lock.acquire()
        lock.release()
        time.sleep(0.3)
        self.sockets.send('status', {'id': self.uid, 'location':'barrel'})
        event, status = self.sockets.recv()
        eq_(status['locations']['barrel']['volume']['value'], 0)
        eq_(status['locations']['bucket']['volume']['value'], 4)
        self.sockets.send('shutdown')

    def test_user_trigger_valve(self):
        f = open(path, 'w')
        f.write('')
        f.close()
        trigger_args = {
            'type': 'user',
            'source': 'barrel',
            'drain_time': 0.5 / 60.0
        }
        self._get_gadgets(trigger_args)
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send(
            'update',
            {
                'bucket': {'volume': {'value': 1, 'units': 'liters'}},
                'barrel': {'volume': {'value': 3, 'units': 'liters'}}
            }
        )
        time.sleep(1)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['valve']['value'], False)
        eq_(self.gadgets.coordinator._state['locations']['barrel']['volume']['value'], 3)
        eq_(self.gadgets.coordinator._state['locations']['bucket']['volume']['value'], 1)
        self.sockets.send('fill bucket', {'units': 'liters', 'value': 1})
        time.sleep(1)
        self.sockets.send('confirmed fill bucket')
        time.sleep(0.2)
        self.sockets.send('status', {'id': self.uid, 'location':'barrel'})
        event, status = self.sockets.recv()
        eq_(status['locations']['barrel']['volume']['value'], 0)
        eq_(status['locations']['bucket']['volume']['value'], 4)
        self.sockets.send('shutdown')


import time
from gadgets.devices.trigger import Trigger


class SwitchTimer(Trigger):

    @property
    def _seconds(self):
        value = self._message.get('value', 0)
        units = self._message.get('units')
        if units == 'hours' or units == 'hour':
            seconds = value * 60.0 * 60.0
        elif units == 'minute' or units == 'minutes':
            seconds = value * 60.0
        elif units == 'second' or units == 'seconds':
            seconds = value
        return seconds

    def wait(self):
        time.sleep(self._seconds)

    def invalidate(self):
        """can't invalidate this trigger"""
        pass


import zmq, json, time
import threading

from zmq.devices import ThreadDevice


class Addresses(object):
    """
    Addresses is a helper class that lets Broker and Sockets easily
    connect to the correct url and port.

    A Gadgets system can be distributed across multiple computers.
    You must decide which instance is the master, and use the default
    'localhost' as the host name.  The subsequent instances can then
    be given the ip address of the master system.  Then all Gadgets
    instances will be able to communicate with each other.
    """

    def __init__(self, host='localhost', in_port=6111, out_port=6112):
        """
        host: the host name of the master gadgets instance
              (the one that is running with localhost as the
              host name)
        in_port: the port number that Socket uses for its
                 subscriber socket
        out_port: the port number that Socket uses for its
                  publisher socket
        """
        self.out_address = 'tcp://{0}:{1}'.format(host, out_port)
        self.in_bind_address = 'tcp://*:{0}'.format(out_port)
        self.in_address = 'tcp://{0}:{1}'.format(host, in_port)
        self.out_bind_address = 'tcp://*:{0}'.format(in_port)

        
class Broker(threading.Thread):
    """
    This is used internally by each Gadgets instance.  You
    shouldn't need to use this class.
    """
    
    def __init__(self, addresses=None):
        """
        addresses: an instance of gadgets.address:Address
        """
        if addresses is None:
            self._addresses = Addresses()
        else:
            self._addresses = addresses
        super(Broker, self).__init__()

    def run(self):
        context = zmq.Context.instance()
        subscriber = context.socket(zmq.SUB)
        subscriber.setsockopt(zmq.SUBSCRIBE, '')
        subscriber.bind(self._addresses.in_bind_address)
        publisher = context.socket(zmq.PUB)
        publisher.bind(self._addresses.out_bind_address)
        run = True
        time.sleep(0.2)
        while run:
            parts = subscriber.recv_multipart()
            if parts[0] ==  'shutdown':
                run = False
            if len(parts) != 2 or parts[1] == 'true' or parts[1] == 'false': #why am I getting these, no one sends them
                continue
            publisher.send_multipart(parts)
        time.sleep(1)
        subscriber.close()
        publisher.close()

        
class Sockets(object):
    """
    Every subclass of Gadget uses this class for all of 
    its communication.  You can also use this class if
    you wish to communicate with the system without 
    a Gadget subclass (on the command line for example).

    Sockets wraps two zeromq sockets and uses them to
    receive incoming messages and send messages out to
    the rest of the system.  It uses zmq.SUB and zmq.PUB
    so each Gadget can control which incoming events
    it receives.  The events argument in __init__ determines
    what events will be received.
    """
    
    def __init__(self, addresses=None, events=[]):
        """
        addresses: an instance of gadgets.address:Address
        events: a list of events to subscribe to.  Only 
                messages that start with the events in
                this list will be received.

                NOTE: If you subscribe to 'update'
                      you will receive messages like
                      'update temperature'
        """
        self.context = zmq.Context.instance()
        if addresses is None:
            addresses = Addresses()
        self.subscriber = self.context.socket(zmq.SUB)
        self.subscriber.connect(addresses.in_address)
        for event in events:
            self.subscriber.setsockopt(zmq.SUBSCRIBE, event)
        self.publisher = self.context.socket(zmq.PUB)
        self.publisher.connect(addresses.out_address)
        time.sleep(0.2)
        
    def send(self, event, message={}):
        """
        The first argument, event, determines what sockets will receive
        the message.  The message argument is any JSON serializable object.
        For example:

        >>> from gadgets import Sockets
        >>> sock = Sockets()
        >>> sock.send('howdy', ['abc'])
        """
        self.publisher.send_multipart([event, json.dumps(message, ensure_ascii=True)])

    def recv(self):
        """
        If a Sockets object that was initialized like this:

        >>> from gadgets import Sockets
        >>> sock = Sockets(events=['howdy'])
        
        sock.recv() will block until some other Sockets instance sends
        a message with 'howdy' as the event.

        >>> event, message = sock.recv()
        >>> print event
        howdy
        """
        parts = self.subscriber.recv_multipart()
        if len(parts) != 2:
            raise Exception(str(parts))
        event, message = parts
        return event, json.loads(message)
        
    def close(self):
        self.publisher.close()
        self.subscriber.close()




from gadgets.io.io import IO

class MotorIOFactory(object):
    """
    Creates a MotorIO object.
    """

    def __init__(self, gpio_a_factory, gpio_b_factory, pwm_factory):
        self._gpio_a_factory = gpio_a_factory
        self._gpio_b_factory = gpio_b_factory
        self._pwm_factory = pwm_factory

    def __call__(self):
        return MotorIO(
            self._gpio_a_factory(),
            self._gpio_b_factory(),
            self._pwm_factory()
        )

class MotorIO(IO):
    """
    Implements gadgets.io.io:IO
    
    This is a compound io object that consists of two gpio pins and
    one pwm.  The gpio pins are used to set the direction the motor
    spins and the PWM is used to set the speed at which it turns.
    This particular class was developed for the Pololu VNH5019 Motor
    Driver:
    
    http://www.pololu.com/catalog/product/1451

    It is possible that it could be used for any motor driver that
    uses two gpios and one pwm.
    """

    def __init__(self, gpio_a, gpio_b, pwm):
        self._gpio_a = gpio_a
        self._gpio_b = gpio_b
        self._pwm = pwm
        self._status = False

    @property
    def status(self):
        return self._status

    def on(self, value):
        value = value.get('value', 100)
        if value < 0:
            self._gpio_b.on()
            self._gpio_a.off()
            self._status = True
        elif value > 0:
            self._gpio_a.on()
            self._gpio_b.off()
            self._status = True
        else:
            self.off()
            
        self._pwm.duty_percent = abs(int(value))

    def off(self):
        self._pwm.off()
        self._gpio_a.on()
        self._gpio_b.on()
        self._status = False

    def close(self):
        self._pwm.close()
        self._gpio_a.close()
        self._gpio_b.close()
        



import curses, time, datetime
from gadgets.ui.window import Window

class Notes(Window):
    """take some notes"""

    _table = '''CREATE TABLE if not exists notes
             (method_id integer, timestamp integer, notes text)'''

    _insert = '''INSERT INTO notes VALUES(?, ?, ?)'''

    def __init__(self, socket, lock, db):
        self._socket = socket
        self._lock = lock
        db.ensure_table(self._table)
        self._db = db
        self._width = 40
        self._height = 18

    def _save(self, notes, method_id):
        timestamp=datetime.datetime.now()
        self._db.save(self._insert, (method_id, int(timestamp.strftime('%s')), notes))

    def __call__(self, screen, data, method_id=None):
        self._lock.acquire()
        win = self._get_subwin(screen)
        self._add_title(win, self.__doc__)
        curses.echo()
        curses.curs_set(1)
        notes = ''
        row = 2
        while not notes.endswith('\n\n'):
            win.move(row ,1)
            notes += win.getstr() + '\n'
            row += 1
        curses.noecho()
        curses.curs_set(0)
        if notes != '':
            self._save(notes, method_id)
        self._lock.release()
        



import time, zmq
import threading
from collections import defaultdict
from gadgets import Gadget

def thermometer_factory(location, name, arguments, addresses):
    if thermometer_factory.server is None:
        thermometer_factory.server = ThermometerServer(addresses, timeout=arguments.get('timeout', 5))
        thermometer_factory.server.start()
    return Thermometer(
        name,
        location,
        addresses,
        uid=arguments['uid'],
        timeout=arguments.get('timeout', 5)
    )

thermometer_factory.server = None

class Thermometer(Gadget):
    """
    When you install Ubuntu or Angstom on a Beaglebone you have access
    to a sysfs interface for dallas 1-wire devices.  Connect a DS18B20+
    temperature sensor to the beaglebone like this::

                          +---------+
                          |         |
                          |  front  |
                          |         |
                          |         |
                          |         |
                          +-+--+--+-+
                            |  |  |
                            |  |  |
                            |  |  |
                            |  |  |
                            |  |  |
                            1  2  3

    Where::

        1 = ground (port 9, pin 1 or 2)
        2 = data   (port 8, pin 6)
        3 = Vcc    (port 9, pin 3 or 4)

    Once you have it connected you can read the temperature from the following
    file::

        $ cat /sys/bus/w1/devices/<unique id>/w1_slave
        b8 01 4b 46 7f ff 08 10 8a : crc=8a YES
        b8 01 4b 46 7f ff 08 10 8a t=27500
    
    Where <unique id> is the embedded uid of the sensor.  Thermometer
    parses the w1_slave file and reports the temperature to the gadgets system.

    To use Thermometer give it a dictionary that maps the location of the
    sensor to its unique id:

    >>> from gadgets import get_gadgets
    >>> from gadgets.pins.beaglebone import pins
    >>> arguments = {
    ...     'locations': {
    ...         'living room': {
    ...             'thermometer': {
    ...                 'type': 'thermometer',
    ...                 'uid': 'uid': '28-0000025f0aba',
    ...              }
    ...          }
    ...      }
    >>> gadgets = get_gadgets(arguments)
    >>> gadgets.start()

    Now the thermometer will start and report its temperature
    to the gadgets system.
    """

    _read_path = '/sys/bus/w1/devices/{0}/w1_slave'

    @property
    def events(self):
        return []

    def __init__(self, location, name, addresses, uid, timeout=5):
        self._uid = uid
        self.value = 0.0
        self._buf = None
        super(Thermometer, self).__init__(location, name, addresses, timeout=timeout)

    @property
    def buf(self):
        if self._buf is None:
            self._buf = open(self._read_path.format(self._uid, 'r'))
        return self._buf

    def event_received(self, event, message):
        s = self.buf.read()
        i = s.find('t=')
        self.buf.seek(0)
        try:
            value = float(s[i+2:]) / 1000.0
        except ValueError:
            pass
        else:
            difference = abs(self.value - value)
            if value >= 0 and difference > 0: #sometimes the dallas thermometers seem to crap out and show an invalid negative value
                self.sockets.send('thermometer server', {'value': value, 'location': self._location})
            self.value = value

    def do_shutdown(self):
        self.buf.close()

class ThermometerServer(Gadget):

    """
    ThermometerServer is intended as a way to consolidate
    the temperature readings and send them all at once
    to the Coordinator.
    """

    def __init__(self, addresses, timeout=5):
        self._values = defaultdict(dict)
        super(ThermometerServer, self).__init__('null', 'thermometer', addresses, timeout=timeout)

    @property
    def events(self):
        return ['thermometer server']

    def event_received(self, event, message):
        if event is None: #a poller timeout
            self.sockets.send('update', self._values)
        else:
            msg = {'value': message['value'], 'units': 'C'}
            self._values[message['location']]['temperature'] = msg



import os
import datetime
import sqlite3

class SQL(object):

    _path = os.path.join(os.path.expanduser('~'), '.gadgets')
    _db_path = os.path.join(_path, 'db')

    def __init__(self):
        self._make_connection()

    def ensure_table(self, command):
        self.connection.execute(command)
        
    def _make_connection(self):
        if not os.path.exists(self._path):
            os.mkdir(self._path)
        self.connection = sqlite3.connect(self._db_path)

    def save(self, insert, args):
        self.connection.execute(insert, args)
        self.connection.commit()

        
class Mongo(object):

    def __init__(self, host, port):
        self._connection = Connection(host, port)
        self._db = self._connection['gadgets']
        
            
        
    
    














import curses, time
from gadgets.ui.window import Window

class Alert(Window):
    """ """

    def __init__(self, label, message):
        self._message = message
        self._label = label
        self._width = 40
        self._height = 4

    def __call__(self, screen):
        win = self._get_subwin(screen)
        self._add_title(win, self._label)
        win.addstr(2, 1, self._message)
        x = win.getstr()
        


import operator
import struct
from gadgets import Gadget
from gadgets.io import GPIO
try:
    from pyctu.commands.api import API
    from pyctu.commands.api_packet import APIPacket
except ImportError:
    API = None
    APIPacket = None

    
class XBeeServer(Gadget):
    """
    The remote XBee modules can be controlled by sending api packets from
    a module that is connected to gadgets system.  A XBeeServer is needed
    because only its serial interface can only be used by one process/thread.
    """

    _XBee_Class = API #this may be switched to something else for testing
    _address_struct = struct.Struct('>Q')

    def __init__(self, location, name, addresses, device):
        self._device = device
        self._highest_channel = 0
        self._state = 0
        self._xbee = None
        self._shift_registers = None
        super(XBeeServer, self).__init__(location, name, addresses)

    def event_received(self, event, message):
        """
        This is the entry point for each new message.
        """
        channel = message['channel']
        address = self._address_struct.pack(message['address'])
        value = '\x05' if message['value'] else '\x04'
        packet = APIPacket(address)
        self.xbee.send('D{0}'.format(channel), value, packet)

    def do_shutdown(self):
        if self._xbee is not None:
            self._xbee.close()

    @property
    def events(self):
        return ['xbee server']

    @property
    def xbee(self):
        if self._xbee is None:
            self._xbee = self._XBee_Class(self._device, 9600)
        return self._xbee




import time, threading, random, uuid
from nose.tools import eq_, raises
from gadgets import Gadgets, Addresses, Sockets, Broker
from gadgets.devices.device import Device
from gadgets.errors import GadgetsError

class GadgetTester(Device):

    def run(self):
        self._register()
        print('shut down')
        self.sockets.send('shutdown', {})

class TestGadgets(object):

    def setup(self):
        port = random.randint(5000, 50000)
        self.addresses = Addresses(in_port=port, out_port=port + 1)
        self.gadgets = Gadgets([], self.addresses)

    def test_create(self):
        pass

    def test_remote_register(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        gt = GadgetTester('garage', 'opener', self.addresses)
        gt.start()
        time.sleep(0.3)
        eq_(self.gadgets.coordinator._ids, ['garage opener'])

    def test_error(self):
        sockets = Sockets(self.addresses)
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        try:
            raise GadgetsError('this is my error', sockets)
        except GadgetsError:
            pass
        eq_(self.gadgets.coordinator._state['errors'], ['this is my error'])
        sockets.send('shutdown')
        sockets.close()

    def test_status(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.gadgets.coordinator._state['locations']['back yard'] = {'sprinklers': {'value': True}}
        uid = 'back yard sprinklers'
        sockets = Sockets(self.addresses, events=[uid])
        time.sleep(0.2)
        msg = {'id': uid}
        sockets.send('status', msg)
        event, message = sockets.recv()
        sockets.send('shutdown')
        eq_(event, uid)
        eq_(message, {u'errors': [], u'locations': {u'back yard': {u'sprinklers': {u'value': True}}}, u'method': {}})




import operator
from gadgets import Gadget
from gadgets.io import GPIO
try:
    from spi import SPI
except ImportError:
    SPI = None

    
class ShiftRegisterServer(Gadget):
    """
    If a gadgets system has multiple ShiftRegisterSwitch objects
    you don't want them conflicting with each other when writing
    to the register's spi interface.  ShiftRegisterServer takes care
    of all spi writes and eliminates any conflicts.
    """

    _SPI_Class = SPI #this may be switched to something else for testing

    def __init__(self, location, name, addresses):
        self._highest_channel = 0
        self._state = 0
        self._spi = None
        self._shift_registers = None
        super(ShiftRegisterServer, self).__init__(location, name, addresses)

    def event_received(self, event, message):
        """
        This is the entry point for each new message.
        """
        channel = message['channel']
        self._toggle_bit(message['value'], channel)
        values = self._get_bytes()
        self.spi.writebytes(values)

    def do_shutdown(self):
        self._state = 0
        values = self._get_bytes()
        self.spi.writebytes(values)

    def register_switch(self, channel):
        """
        If the user configures a system with only two ShiftRegisterSwitch
        objects, but one of them uses, say,  channel 8, then there must be
        two physical shift registers in use with a bunch of unused pins.
        This function allows this server to send the correct number of
        characters to the spi interface (once char per chip).
        """
        if channel >= self._highest_channel:
            self._highest_channel = channel

    @property
    def events(self):
        return ['shift register server']

    @property
    def shift_registers(self):
        """
        return the number of shift registers
        """
        if self._shift_registers is None:
            self._shift_registers = ((self._highest_channel) / 8) + 1
        return self._shift_registers

    def _get_bytes(self):
        """
        divide self._state into 8 bit values, for example:

        0b11100000011  ->   [0b111, 0b11]
        """
        bytes_list = [(self._state >> (i * 8)) & 255 for i in xrange(self.shift_registers)]
        bytes_list.reverse()
        return bytes_list

    def _toggle_bit(self, value, channel):
        """
        if value is True and channel is 3, then:
        0b00000000
        goes to:
        0b00001000
        """
        if value:
            self._state |= 1 << channel
        else:
            self._state &= ~(1 << channel)
        
    @property
    def spi(self):
        if self._spi is None:
            self._spi = self._SPI_Class(2, 0)
            self._spi.msh = 1000000 #the python module is limited in how fast it can send
        return self._spi



import threading, time, zmq
from gadgets.sockets import Sockets


class Gadget(threading.Thread):
    """
    The components that run in the gadgets system should be a subclass
    of Gadget.  

    After a gadget starts and registers, it listens for zmq messages
    that are specified by the events attribute.  (see the gadgets.sockets
    module for more details).  After a message is received the 'event_received'
    method is called.
    
    There are many possible uses of Gadget subclasses.  One of the main
    subclasses built into Gadgets is Device.  It is meant for controlling
    physical devices.  A Gadget subclass isn't just for interfacing with
    hardware.  For example, here is a class that records all of the goings
    on of the system::

    >>> class Recorder(Gadget):
    ...     @property
    ...        def events(self):   
    ...            return ['UPDATE']
    ...
    ...     def event_received(self, event, message):
    ...         with open('/tmp/gadgets_data.txt', 'a') as buf:
    ...             buf.write(json.dumps(message))

    It would probably better to make one that saves the messages to
    a database.
    
    In order to make a Gadgets subclass, you must define an events
    property and implement event_received.  The event_received
    will only receive messages that you specified in the events
    property.
    """

    def __init__(self, location, name, addresses, timeout=None):
        self._timeout = timeout
        self._addresses = addresses
        self._location = location
        self._name = name
        self._id = '{0} {1}'.format(self._location, self._name)
        self._registration_event = '{0} registration'.format(self._id)
        self._sockets = None
        self._events_cache = None
        super(Gadget, self).__init__()

    @property
    def uid(self):
        """
        A gadget uid us composed of the location and name
        properties.  No two Gadget instances should have
        the same uid.
        """
        return '{0} {1}'.format(self._location, self._name)

    @property
    def events(self):
        """
        Returns a list of events that will trigger the event_received method
        """
        raise NotImplemented

    def event_received(self, event, message):
        """
        receives the events defined by self.events
        """
        raise NotImplemented

    def do_shutdown(self):
        """
        override this if your subclass needs to do more stuff
        in order to shutdown cleanly.
        """
        pass

    def run(self):
        self._register()
        self._stop = False
        if self._timeout is not None:
            poller = zmq.Poller()
            poller.register(self.sockets.subscriber, zmq.POLLIN)
        while not self._stop:
            if self._timeout:
                socks = dict(poller.poll(timeout=self._timeout * 1000))
                if self.sockets.subscriber in socks and socks[self.sockets.subscriber] == zmq.POLLIN:
                    event, message = self.sockets.recv()
                else:
                    event, message = None, None
            else:
                event, message = self.sockets.recv()
            self._do_event_received(event, message)
        self.sockets.close()
        self.do_shutdown()

    def _do_event_received(self, event, message):
        if event == 'status':
            self.sockets.send(str(message['id']), self.status)
        elif event == 'shutdown':
            self._stop = True
        else:
            self.event_received(event, message)

    @property
    def status(self):
        """Device overrides this property to indicate weather or not
        the Device is turned on"""
        return False

    @property
    def _events(self):
        if self._events_cache is None:
            self._events_cache = ['status', 'shutdown', self._id, self._registration_event] + self.events
        return self._events_cache

    @property
    def sockets(self):
        if self._sockets is None:
            self._sockets = Sockets(self._addresses, events=self._events)
        return self._sockets

    def _register(self):
        self.sockets.send(
            'register',
            {
                'location': self._location,
                'name': self._name,
            }
        )
        



from gadgets.devices.switch.switch import Switch
from gadgets.devices.switch.xbee.io import XBeeIOFactory
from gadgets.devices.switch.xbee.server import XBeeServer


def xbee_factory(location, name, arguments, addresses, io_factory=None):
    if xbee_factory.server is None:
        """
        """
        xbee_factory.server = XBeeServer('', 'shift register server', addresses, arguments['device'])
        xbee_factory.server.start()
    channel = arguments['channel']
    address = arguments['address']
    return Switch(
        location=location,
        name=name,
        addresses=addresses,
        io_factory=XBeeIOFactory(addresses, channel, address),
        on=arguments.get('on'),
        off=arguments.get('off'),
        momentary=arguments.get('momentary', False)
    )

xbee_factory.server = None




from gadgets.devices.valve.triggers import FloatTriggerFactory
from gadgets.devices.valve.valve import Valve
from gadgets.devices.valve.triggers import GravityTriggerFactory, GravityTrigger, TimerTriggerFactory, UserTriggerFactory
from gadgets.io import GPIOFactory
from gadgets.errors import GadgetsError

def get_float_trigger_factory(location,arguments, addresses):
    return FloatTriggerFactory(location, arguments['pin'], arguments['volume'], addresses)

def get_gravity_trigger_factory(location, arguments, addresses):
    try:
        kw = dict(
            location=location,
            addresses=addresses,
            source=arguments['source'],
            tank_radius=arguments['tank_radius'],
            valve_radius=arguments['valve_radius'],
            valve_coefficient=arguments['valve_coefficient']
        )
    except KeyError:
        raise GadgetsError(GravityTriggerFactory.__init__.__doc__)
    return GravityTriggerFactory(**kw)

def get_timer_trigger_factory(location, arguments, addresses):
    try:
        kw = dict(
            location=location,
            addresses=addresses,
            source=arguments['source'],
            drain_time=arguments['drain_time'],
        )
    except KeyError:
        raise GadgetsError(TimerTriggerFactory.__init__.__doc__)
    return TimerTriggerFactory(**kw)

def get_user_trigger_factory(location, arguments, addresses):
    try:
        kw = dict(
            location=location,
            addresses=addresses,
            source=arguments['source'],
        )
    except KeyError:
        raise GadgetsError(UserTriggerFactory.__init__.__doc__)
    return UserTriggerFactory(**kw)

class ValveFactory(object):

    gpio_factory = GPIOFactory
    
    def __call__(self, location, name, arguments, addresses):
        kw = dict(
            location=location,
            name=name,
            addresses=addresses,
            io_factory=self.gpio_factory(arguments['pin'], direction='out'),
        )
        trigger_type = arguments['trigger']['type']
        if trigger_type == 'float':
            trigger_factory = get_float_trigger_factory(location, arguments['trigger'], addresses)
        if trigger_type == 'gravity':
            trigger_factory = get_gravity_trigger_factory(location, arguments['trigger'], addresses)
        if trigger_type == 'timer':
            trigger_factory = get_timer_trigger_factory(location, arguments['trigger'], addresses)
        if trigger_type == 'user':
            trigger_factory = get_user_trigger_factory(location, arguments['trigger'], addresses)
        kw['trigger_factory'] = trigger_factory
        return Valve(**kw)



import threading, time
from gadgets import Addresses, GadgetsFactory, Sockets
from gadgets.devices import Switch, Valve
from gadgets.pins.beaglebone import pins
from nose.tools import eq_

class TestGadgetsFactory(object):

    def setup(self):
        self.addresses = Addresses()
        self.factory = GadgetsFactory(self.addresses)

    def test_create(self):
        pass

    def test_create_switch(self):
        args = {
            'locations': {
                'fish tank': {
                    'light': {
                        'type': 'switch',
                        'pin': pins['gpio'][8][12]
                    }
                }
            }
        }
        gadgets = self.factory(args)
        devices = gadgets._gadgets
        eq_(len(devices), 1)
        assert isinstance(devices[0], Switch)

    def test_create_float_valve(self):
        args = {
            'locations': {
                'fish tank': {
                    'valve': {
                        'type': 'valve',
                        'pin': pins['gpio'][8][12],
                        'trigger': {
                            'type':'float',
                            'pin': pins['gpio'][8][11],
                            'volume': {
                                'value': 5.5,
                                'units': 'gallons',
                            }
                        }
                    }
                }
            }
        }
        gadgets = self.factory(args)
        devices = gadgets._gadgets
        eq_(len(devices), 1)
        t = threading.Thread(target=gadgets.start)
        t.start()
        time.sleep(1)
        eq_(dict(gadgets.coordinator._external_events), {u'fish tank': {u'valve': {'on': u'fill fish tank', 'off': u'stop filling fish tank'}}})
        sockets = Sockets()
        sockets.send('shutdown')
        assert isinstance(devices[0], Valve)
        sockets.close()



from gadgets.io import GPIOFactory
from gadgets.devices.cooler.cooler import Cooler
from gadgets.devices.heater.triggers.temperature import TemperatureTriggerFactory

def cooler_factory(location, name, arguments, addresses, io_factory=None, trigger_factory=None):
    comparitor = lambda x, y: x <= y
    if io_factory is None:
        io_factory = GPIOFactory(arguments['pin'], direction='out')
    if trigger_factory is None:
        trigger_factory=TemperatureTriggerFactory(location, addresses, comparitor=comparitor)
    kw = dict(
        location=location,
        name=name,
        addresses=addresses,
        io_factory=io_factory,
        trigger_factory=trigger_factory
    )
    if 'on' in arguments:
        kw['on'] = arguments['on']
    if 'off' in arguments:
        kw['off'] = arguments['off']
    return Cooler(**kw)
        



import curses, time, os, glob
from gadgets.ui.window import Window

class GetFile(Window):
    """ """

    def __init__(self, label):
        self._label = label
        self._width = 40
        self._height = 4

    def _browse(self):
        path = os.getcwd()
        i = 0
        while True:
            self._win.addstr(2, 1, path)
            x = self._win.getch()
            if x == 10:
                return path
            elif x == 127: #backspace
                if path.endswith('/'):
                    path = path[:-1]
                self._win.addstr(2, 1, ' ' * (len(path) + 1))
                path, junk = os.path.split(path)
                path += '/'
            elif x == 9: #tab
                path, junk = os.path.split(path)
                contents = glob.glob(os.path.join(path, '{0}*'.format(junk)))
                if len(contents) > 0:
                    path = contents[0]
            else:
                path += chr(x)
            if x == ord('/'):
                i = 0

    def __call__(self, screen):
        self._win = self._get_subwin(screen)
        self._add_title(self._win, self._label)
        return self._browse()


        


from gadgets.devices.switch.shift_register.factory import shift_register_switch_factory
from gadgets.devices.switch.gpio.factory import switch_factory
from gadgets.devices.switch.xbee.factory import xbee_factory



import time, threading, random, uuid, tempfile, os
from nose.tools import eq_
from gadgets import Addresses, get_gadgets, Sockets
from gadgets.sensors import Thermometer

class TestThermometer(object):

    def setup(self):
        port = random.randint(5000, 50000)
        self.addresses = Addresses(in_port=port, out_port=port + 1)
        self.sockets = None
        self.thermometer = Thermometer('livingroom', 'thermometer', self.addresses, uid='x')

    def teardown(self):
        if self.sockets is not None:
            self.sockets.send('shutdown')
            time.sleep(0.2)
            self.sockets.close()

    def test_create(self):
        pass

    def test_run(self):
        tmp = tempfile.mkdtemp()
        os.mkdir(os.path.join(tmp, 'x'))
        path = os.path.join(tmp, 'x', 'w1_slave')
        f = open(path, 'w')
        f.write('t=21500')
        f.close()
        Thermometer._read_path = '{0}/x/w1_slave'.format(tmp)
        arguments = {
            'locations': {
                'living room': {
                    'thermometer': {
                        'type': 'thermometer',
                        'uid': 'x',
                        'timeout': 0.5,
                    }
                }
            }
        }
        gadgets = get_gadgets(arguments, self.addresses)
        t = threading.Thread(target=gadgets.start)
        t.start()
        time.sleep(2)
        self.sockets = Sockets(self.addresses, events=['test thermometer'])
        self.sockets.send('status', {'id': 'test thermometer'})
        event, message = self.sockets.recv()
        eq_(event, 'test thermometer')
        expected = {u'errors': [], u'locations': {u'thermometer': {u'temperature': {u'units': u'C', u'value': 21.5}}}, u'method': {}}
        eq_(message, expected)

        
        
        



import curses, time
from gadgets.ui.window import Window
from gadgets.ui.dialogs.prompt import Prompt
from gadgets.ui.dialogs.alert import Alert

KEYCODE_ENTER = 10
KEYCODE_ESC = 27

class CommandMode(Window):
    """
    
    """

    def __init__(self, parent):
        self._parent = parent
        self._lock = parent._lock
        self._command_mode = self._get_command_mode()
        self.is_active = False

    def _get_command_mode(self):
        return {
            'sequence':'',
            'keys': [],
        }

    @property
    def keys(self):
        return self._command_mode['keys']

    def __call__(self, x):
        if x == KEYCODE_ESC:
            self._command_mode = self._get_command_mode()
            self.is_active = False
        elif len(self._command_mode['keys']) < 2:
            self._update_command_mode(x)
        elif len(self._command_mode['keys']) == 2 and x == KEYCODE_ENTER:
            self._do_command()
        elif len(self._command_mode['keys']) == 2 and x == ord('a'):
            argument = self._get_argument()
            if argument:
                self._do_command(argument)
        
        
    def _get_argument(self):
        self._lock.acquire()
        p = Prompt('value', str)
        argument = p(self._parent._screen)
        try:
            val, units = argument.split(' ')
        except ValueError:
            a = Alert('Error', 'enter a value and its units')
            a(self._parent._screen)
            return_value = None
        else:
            return_value = {'value': float(val), 'units': units}
        self._lock.release()
        return return_value

    def _do_command(self, argument={}):
        keys = self._command_mode['keys']
        d = self._parent._data['locations']
        for name in self._command_mode['keys']:
            d = d[name]
        status = 'off' if d['value'] else 'on'
        command = self._parent._getter.commands[keys[0]][keys[1]][status]
        self._parent.sockets.send(str(command), argument)

    def _update_command_mode(self, x):
        self._command_mode['sequence'] += chr(x)
        sequence = self._command_mode['sequence']
        d = self._parent._data['locations']
        for name in self._command_mode['keys']:
            d = d[name]
        keys = [key for key in d.keys() if key.startswith(sequence)]
        if len(keys) == 1:
            self._command_mode['keys'].append(keys[0])
            self._command_mode['sequence'] = ''


import threading, zmq, time, json, sys, uuid
from gadgets import Addresses, Sockets

class DataGetter(threading.Thread):
    
    def __init__(self, target, addresses):
        self.uid = str(uuid.uuid1())
        self._addresses = addresses
        self._stop_event = threading.Event()
        self.commands = {}
        super(DataGetter, self).__init__()
        self._target = target
        self._sockets = None
        self.locations = None

    def do_shutdown(self):
        self._stop_event.set()

    def run(self):
        self.main_loop()

    def main_loop(self):
        self.sockets.send('status', {'id': self.uid})
        event, status = self.sockets.recv()
        self.locations = status['locations'].keys()
        self.sockets.send('events', {'id': self.uid})
        _id, self.commands = self.sockets.recv()
        self._target(status)
        poller = zmq.Poller()
        poller.register(self.sockets.subscriber, zmq.POLLIN)
        while not self._stop_event.isSet():
            self._get_data(poller)
        self.sockets.close()

    def _get_data(self, poller):
        socks = dict(poller.poll(1000))
        if self.sockets.subscriber in socks and socks[self.sockets.subscriber] == zmq.POLLIN:
            event, msg = self.sockets.recv()
            if not self._target(msg):
                self._stop_event.set()
                return

    def stop(self):
        self._stop_event.set()

    @property
    def sockets(self):
        if self._sockets is None:
            self._sockets = Sockets(self._addresses, events=['UPDATE', self.uid])
        return self._sockets

def print_data(data):
    return True


if __name__ == '__main__':
    host = sys.argv[1]
    addresses = Addresses(host=host)
    dg = DataGetter(print_data, addresses)
    dg.start()




import time, uuid
from gadgets.devices.trigger import Trigger
from gadgets.devices.valve.triggers import TimerTriggerFactory, TimerTrigger
from gadgets import Sockets

class UserTriggerFactory(TimerTriggerFactory):

    def __call__(self, on_event, off_event, message, target):
        uid = str(uuid.uuid1())
        sockets = Sockets(self._addresses, events=[uid])
        sockets.send('status', {'id': uid})
        returned_uid, message = sockets.recv()
        source_volume = message['locations'][self._source].get('volume', {}).get('value', 0.0)
        volume = message['locations'][self._location].get('volume', {}).get('value', 0.0)
        sockets.close()
        return UserTrigger(
            self._location,
            on_event,
            off_event,
            message,
            self._addresses,
            target,
            volume,
            self._source,
            source_volume,
        )
        

class UserTrigger(TimerTrigger):

    def __init__(self, location, on_event, off_event, message, addresses, target, volume, source, source_volume, drain_time=None):
        super(UserTrigger, self).__init__(location, on_event, off_event, message, addresses, target, volume, source, source_volume, drain_time=None)
        self._confirmed_event = 'confirmed {0}'.format(on_event)
        self._events = [self._confirmed_event]

    def wait(self):
        event, message = self.sockets.recv()
        if event != self._confirmed_event:
            self.wait()





from gadgets import Gadget
from gadgets.rcl import parse_command

class Device(Gadget):
    """
    Device is what should be subclassed if you want a Gadget that
    controls a physical device.

    Each device has an io attribute.  The object represented by io
    only has to have an on and off method.  The built in devices
    use io such as GPIO, PWM, and Poller.

    When a Device is turned on with arguments, it should start up
    a trigger.  A trigger is a threaded class that starts up and
    waits for some event to happen.  When the event happens it does
    two things: turns off the io object and notifies the reset of
    the system with a 'completed' event.

    For example, if you have a gadgets system for watering the grass
    you could send a RCL command like this:

    >>> from gadgets import Sockets
    >>> sock = Sockets()
    >>> sock.send('turn on front yard sprinkers')

    This is an example of an RCL command that has no arguments.  If
    this command was send instead:

    >>> sock.send('turn on front yard sprinkers for 15 minutes')

    the command is considered to have arguments.  If the Device that
    controls the front yard sprinklers is a Switch, then its trigger
    is a GPIOTimer.  A GPIOTimer would be started up and after 15
    minutes it would turn off the GPIO of the Switch that started
    it.
    """

    _units = []
    _on_template = 'turn on {location} {name}'
    _off_template = 'turn off {location} {name}'

    def __init__(self, location, name, addresses, io_factory=None, trigger_factory=None, on=None, off=None):
        super(Device, self).__init__(location, name, addresses)
        self._on_event = self.get_on_event(on)
        self._off_event = self.get_off_event(off)
        self._off_trigger = None
        self._io_factory = io_factory
        if trigger_factory is not None:
            self._trigger_factory = trigger_factory
        self._io = None

    @property
    def events(self):
        """
        A list of events that will trigger the event_received method
        """
        return [self._on_event, self._off_event]

    @property
    def status(self):
        return self.io.status

    def get_on_event(self, on=None):
        """
        Unless the user passes in the on event or a subclass
        overrides this message, then this device turns on
        in response to this message.  So if this is a garage
        door opener, self._location is garage, self._name is
        opener.  The default on event will be:

        'turn on garage door'

        You might want to pass in the on event manually

        on='open {location}'

        so that it becomes

        'open garage door'
        """
        if on:
            on = on.format(location=self._location, name=self._name)
        else:
            on = self._on_template.format(location=self._location, name=self._name)
        return on

    def get_off_event(self, off=None):
        """see _get_on_event"""
        if off:
            off = off.format(location=self._location, name=self._name)
        else:
            off = self._off_template.format(location=self._location, name=self._name)
        return off

    def event_received(self, event, message):
        """
        Called whenever one of the events returned by self.events
        is received.
        """
        if event.startswith(self._on_event):
            if len(event) > len(self._on_event):
                event, message = parse_command(event, message)
            self.on(message)
        elif event.startswith(self._off_event):
            self.off(message)
        
    @property
    def io(self):
        """
        Every device must have an io property.  Switch uses a GPIO,
        ElectricHeater uses PWM.  ElectricMotor uses GPIO and PWM.
        The object returned here must follow the API defined at
        
        gadgets.io.io:IO
        """
        if self._io is None:
            self._io = self._io_factory()
        return self._io

    def on(self, message):
        """
        Gets called by event_received in the case of an on event
        being received.  It calls on of self.io and starts a trigger.
        It also calls self._update_status to notify the whole gadgets
        system of a state change.
        """
        if not self.io.status:
            self.io.on()
            if self._should_get_trigger(message):
                self._invalidate_trigger()
                self._off_trigger = self._get_trigger(message)
                self._off_trigger.start()
            self._update_status(True)

    def off(self, message=None):
        """
        Gets called by event_received in the case of an off event.
        If a trigger is currently running it invalidates the trigger.
        It also calls self._update_status to notifiy the whole gadgets
        system of a state change.
        """
        if self.io.status:
            self.io.off()
            self._invalidate_trigger()
            self._update_status(False, message)

    def _should_get_trigger(self, message):
        return message.get('units') in self._units
        
    def _get_trigger(self, message):
        return self._trigger_factory(
            self._on_event,
            self._off_event,
            message,
            self.off
        )
        
    def _invalidate_trigger(self):
        if self._off_trigger is not None and self._off_trigger.is_alive():
            self._off_trigger.invalidate()
            self._off_trigger = None

    def _get_status_message(self, status, message=None):
        outgoing_message = {
            self._location: {
                self._name: {
                    'value': status
                }
            }
        }
        if message is not None:
            for location in message:
                for name, val in message[location].iteritems():
                    if location not in outgoing_message:
                        outgoing_message[location] = dict()
                    outgoing_message[location][name] = {
                        'value': val['value'],
                        'units': val['units']
                    }
        return outgoing_message
                                
    def _update_status(self, status, message=None):
        self.sockets.send(
            'update',
            self._get_status_message(status, message)
        )

    def _register(self):
        self.sockets.send(
            'register',
            {
                'location': self._location,
                'name': self._name,
                'on': self._on_event,
                'off': self._off_event,
                'uid': self.uid,
            }
        )

    def do_shutdown(self):
        if self._io is not None:
            self.io.close()




from gadgets.io import GPIOFactory
from gadgets.devices.switch.switch import Switch


def switch_factory(location, name, arguments, addresses, io_factory=None):
    if io_factory is None:
        io_factory = GPIOFactory(arguments['pin'], direction='out')
    return Switch(
        location=location,
        name=name,
        addresses=addresses,
        io_factory=io_factory,
        on=arguments.get('on'),
        off=arguments.get('off'),
        momentary=arguments.get('momentary', False)
    )



from gadgets.io.poller import Poller
import threading

class Counter(object):
    """
    Blocks for a given number of rising edges on a gpio pin.
    """
    
    def __init__(self, pin, ticks):
        """
        pin: a gpio pin from gadgets.pins
        ticks: the number of rising edges for which to wait
        """
        self._ticks = int(ticks)
        self._poller = Poller(pin)

    def wait(self):
        """
        Blocks until for the number of rising
        edges specified by self._ticks
        """
        for i in xrange(self._ticks):
            self._poller.wait()
        self._poller.close()




import math, datetime, zmq


class VolumeConverter:
    """
    t = ((3.14 x dia^2) / (C x A)) x (h / (8 x g))^0.5

    Where
    t = time in seconds
    dia = diameter of tank in ft
    C = Orface coef. (0.7 to 0.91) I use 0.81 for a pipe penitrating the tank wall.
    A = aera of orface in sf
    h = height of water above outlet in ft
    g = gravitational constant (32.2)

    """
    def __init__(self, tank_radius, valve_radius, coefficient=0.81):
        tank_radius = tank_radius * 2.54  # convert to centimeters
        valve_radius = valve_radius * 2.54 # convert to centimeters
        self.coefficient = coefficient
        self.tank_area = math.pi * tank_radius ** 2
        valve_area = math.pi * valve_radius ** 2
        g = 9.806 * 100.0 #centimeters
        x = pow((2 / g), 0.5)
        self.k = (self.tank_area * x) / (valve_area * coefficient)
        self.valve_area = valve_area
        self.x = x

    def get_volume(self, starting_volume, time):
        starting_volume = starting_volume * 1000.0 #convert to cubic centimeters
        height = self._get_height(starting_volume)
        dh = math.fabs(((time / self.k) ** 2) - (2 * (time / self.k) * pow(height, 0.5)))
        return (self.tank_area * dh) / 1000.0
    
    def get_drain_time(self, starting_volume, volume):
        volume = 1000 * volume #convert to cubic centimeters
        starting_volume = starting_volume * 1000.0
        height_diff = self._get_height(volume)
        height = self._get_height(starting_volume)
        h2 = height - height_diff
        return (pow(height, 0.5) - pow(h2, 0.5)) * self.k

    def get_coefficient(self, starting_volume, volume, drain_time):
        hi = self._get_height(starting_volume * 1000.0)
        dh = self._get_height(volume * 1000.0)
        hf = hi - dh
        At = self.tank_area
        Av = self.valve_area
        t = drain_time
        x = self.x
        return ((At * x) / Av) * ((hi**0.5) - (hf**0.5)) / t

    def _get_height(self, volume):
        return volume / self.tank_area

def calibrate():
    vc = VolumeConverter(7.5, 0.1875)
    addr = 'tcp://brewery:5000'
    socket = context.socket(zmq.PUSH)
    socket.connect(addr)
    socket.send('fill tun 4')
    start = datetime.datetime.now()
    x = raw_input('Push enter when 2 liters has dispensed')
    end = datetime.datetime.now()
    socket.send('stop filling tun')
    delta = end - start
    t = delta.seconds + delta.microseconds/1000000.0
    coefficient = vc.get_coefficient(26.5, 2.0, t)





from gadgets.devices.switch.shift_register.io import ShiftRegisterIO

class XBeeIOFactory(object):
    
    def __init__(self, addresses, channel, address):
        self._addresses = addresses
        self._channel = channel
        self._address = address
    
    def __call__(self):
        return XBeeIO(self._addresses, self._channel, self._address)


class XBeeIO(ShiftRegisterIO):

    """
    Implements the gadgets.io interface for XBee switches.

    Why XBee?  Gadget systems can be distributed across many
    BeagleBones or Raspberry Pis.  However if all you need
    is some remote Switch objects, it may be cheaper to use
    xbee radios.  XBee series 2 modules have 9 gpio pins each, and
    by using API mode you can control multiple remote XBee modules
    with a single XBee coordinator connected to your Beaglebone
    or Raspberry Pi.  The XBee modules do not need to be connected
    to a microcontroller, so all you have to do is provide power.  
    That should be less expensive than buying a Raspberry Pi along with
    a power supply, sd card and WiFi dongle.

    Here is an example::

       >>> from gadgets import get_gadgets
       >>> arguments = {
       ...     "locations": {
       ...         "back yard": {
       ...             "gnome": {
       ...                 "type": "xbee",
       ...                 "channel": 3,
       ...                 "address": 0x0013A200409825C1,
       ...                 "device": '/dev/ttyUSB0',
       ...            }
       ...         }
       ...     }
       ... }
       >>> gadgets = get_gadgets(arguments)

    Address is the address of the XBee.  One way to find the address is
    to look at the label on the bottom of the module.  It will look
    something like this::

         ---- 004-revE
        |bar |0013A200
        |code|409825C1
         ----

    You combine the middle line (address high) with the bottom line (
    address low) to come up with the address that you use as an argument
    in the example above.

    Before any of the above will work, you need two XBee series 2
    modules - one connected to the BeagleBone or Raspberry Pi, and one
    one that is connected to the physical devices that you wish to
    control.  The one connected to the gadgets system must be running
    the 'coordinator api' firmware and the remote XBee(s) must be
    running 'router at' firmware.  They must also be configured with the
    same pan id.
    
    Now you can turn it on by creating a socket and giving the switches
    a RCL command::

    >>> from gadgets import Sockets
    >>> s = Sockets()
    >>> s.send("turn on back yard gnome")

    NOTE:  XBeeIO depends on pyctu for its XBee API mode communication::

        $ easy_install pyctu
    
    """

    def __init__(self, addresses, channel, address):
        self._address = address
        super(XBeeIO, self).__init__(addresses, channel)
    
    def _send(self, value):
        self._sockets.send(
            'xbee server',
            {'value': value, 'channel': self._channel, 'address': self._address}
        )
        self._status = value




import time

from gadgets.gadget import Gadget
from gadgets.sockets import Sockets, Addresses, Broker
from gadgets.coordinator import Coordinator

from gadgets.devices.cooler.cooler_factory import cooler_factory
from gadgets.devices.heater.electric_heater_factory import electric_heater_factory
from gadgets.devices.switch import switch_factory, shift_register_switch_factory, xbee_factory
from gadgets.devices.motor.motor_factory import motor_factory
from gadgets.devices.button.factory import button_factory
from gadgets.devices.valve.valve_factory import ValveFactory

from gadgets.sensors import thermometer_factory


def get_gadgets(arguments, addresses=None):
    factory = GadgetsFactory(addresses=addresses)
    return factory(arguments)


class GadgetsFactory(object):

    def __init__(self, addresses=None, ensure_off=False):
        self._ensure_off = ensure_off
        if addresses is None:
            addresses = Addresses()
        self._addresses = addresses
        self._factories = {
            'valve': ValveFactory(),
            'switch': switch_factory,
            'xbee': xbee_factory,
            'shift register switch': shift_register_switch_factory,
            'motor': motor_factory,
            'electric heater': electric_heater_factory,
            'cooler': cooler_factory,
            'thermometer': thermometer_factory,
            'button': button_factory
        }

    def add_factory(self, name, factory):
        self._factories[name] = factory

    def __call__(self, gadget_arguments):
        output = []
        for location, arguments in gadget_arguments['locations'].iteritems():
            for name, device_arguments in arguments.iteritems():
                if 'type' not in device_arguments:
                    raise GadgetsError('type needed in arguments {0}'.format(device_arguments))
                factory = self._factories[device_arguments['type']]
                gadget = factory(location, name, device_arguments, self._addresses)
                output.append(gadget)
        return Gadgets(output, self._addresses, ensure_off=self._ensure_off)

class Gadgets(object):
    """
    Gadgets holds all the Gadget subclass instances and starts them
    up.  If this instance is configured as the master (by using
    'localhost' as the host name in Addresses), then it also starts
    up a Broker and a Coordinator.
    """

    def __init__(self, gadgets, addresses, ensure_off=False):
        self._gadgets = gadgets
        self._ensure_off = ensure_off
        self._addresses = addresses
        self.coordinator = None
        super(Gadgets, self).__init__()

    def start(self):
        if self._is_master:
            broker = Broker(self._addresses)
            broker.start()
            time.sleep(0.2)
            self.coordinator = Coordinator(self._addresses)
            self.coordinator.start()
        for gadget in self._gadgets:
            gadget.start()
        if self._is_master:
            self.coordinator.join()
        else:
            gadget.join()

    @property
    def _is_master(self):
        return 'localhost' in self._addresses.out_address



"""
GPIOFactory
GPIO
"""
import os
from gadgets.io.io import IO
from gadgets.errors import GadgetsError

class GPIOFactory(object):
    """
    Creates GPIO objects
    """
    def __init__(self, pin, direction='out', pullup=False):
        self._pin = pin
        self._direction = direction
        self._pullup = pullup

    def __call__(self):
        return GPIO(self._pin, self._direction, self._pullup)


class GPIO(IO):
    """
    Turning on a pin using the linux sysfs interface requires you to:
    
    * make sure the mux mode of the pin is set to gpio (which requires you to find out the name of the pin)
    * figure out the number to write to /sys/class/gpio/export in order to enable the gpio interface
    * write 'out' to /sys/class/gpio/gpio<gpio number>/direction
    * write 1 to /sys/class/gpio/gpio<gpio number>/value

    All this just to turn on a pin.  This GPIO class takes care of all
    that for you.  All you have to do is pick the port and pin (port 8,
    pin 3 in this case) and initialize a GPIO object like:

        >>> from gadgets.pins.beaglebone import pins
        >>> from gadgets.io import GPIO
        >>> gpio = GPIO(pins['gpio'][8][3])

    Now you can turn the pin on and off by:
    
        >>> gpio.on()
        >>> gpio.off()

    If you want to use a gpio pin as input, you initialize it like this:

        >>> from gadgets.pins.beaglebone import pins
        >>> gpio = GPIO(pins['gpio'][8][11], direction='in')

    You can then read the value of the pin by:

        >>> print gpio.value
        0
    """

    _path = '/sys/class/gpio'
    _export_path = '/sys/class/gpio/export'
    _gpio_path = '/sys/class/gpio/gpio{0}'
    _base_path = '/sys/class/gpio/gpio{0}/{1}'
    _mux_path = '/sys/kernel/debug/omap_mux'

    def __init__(self, pin, direction='in', pullup=False):
        """
        mux: the name of the pin, as appears in omap_mux
        export: the gpio number that is written to export
        direction: 'in' or 'out'
        pullup: if True, configure the gpio to use the
        internal pullup resistor 
        """
        self._mux = pin['mux']
        self._export = pin['export']
        self._direction = direction
        self._pullup = pullup
        self._file_cache = None
        self._initialize_pin()
        self._value = 0
        self._status = False
        

    @property
    def status(self):
        return self._status

    @property
    def _file(self):
        """
        Returns the sysfs file object that is the
        Linux interface to the gpio pin.
        """
        if self._file_cache is None:
            self._file_cache = self._open_file()
        return self._file_cache

    def on(self):
        """
        Sets the gpio pin high.
        """
        if self._direction == 'out':
            self._value = 1
            self._file.write('1')
            self._file.flush()
            self._status = True

    def off(self):
        """
        Sets the gpio pin low.
        """
        if self._direction == 'out':
            self._value = 0
            self._file.write('0')
            self._file.flush()
            self._status = False

    def value(self):
        """
        Get the value of the gpio pin.
        """
        if self._direction == 'out':
            value = self._value
        else:
            value = int(self._file.read())
            self._file.seek(0)
        return value

    def close(self):
        """
        Closes the file that GPIO uses to set/read the
        value of the pin.
        """
        self._file.close()

    def _open_file(self):
        """
        Opens the sysfs gpio interface file that sets
        the pin high or low.
        """
        value_path = self._base_path.format(self._export, 'value')
        if self._direction == 'out':
            buf = open(value_path, 'w')
            buf.write('0')
            buf.flush()
        else:
            buf = open(value_path, 'r')
        return buf

    def _write_to_file(self, path, value):
        """
        writes the given value to a sysfs
        path
        """
        with open(path, 'w') as buf:
            buf.write(value)

    @property
    def mux_path(self):
        """
        creates the full path to the sysfs mux interface
        for this pin.  
        """
        return os.path.join(self._mux_path, self._mux)

    @property
    def home_dir(self):
        """
        After a gpio pin is enabled the sysfs interface for the
        pin is created as a directory.  This property creates
        the path to that directory.
        """
        return self._gpio_path.format(self._export)

    def _get_mux(self):
        """
        Need to find out if the muxing of gpio pins is specific to
        beaglebone.
        """
        bits = [1, 1, 1, 0, self._pullup, self._direction=='in']
        val = 0
        for i, bit in enumerate(bits):
            val |= bit << i
        return hex(val)[2:]

    def _initialize_pin(self):
        """
        Sets the mux value to gpio then enables the
        gpio sysfs inteface by writing the export name
        to the export file.
        """
        self._write_to_file(self.mux_path, self._get_mux())
        if not os.path.exists(self.home_dir):
            self._write_to_file(self._export_path, str(self._export))
        if not os.path.exists(self.home_dir):
            raise GadgetsError(
                'failed gpio export: {0}'.format(
                    self.home_dir))
        path = self._base_path.format(self._export, 'direction')
        self._write_to_file(path, self._direction)



import time, threading
from collections import defaultdict

from gadgets.sockets import Sockets
from gadgets.errors import GadgetsError
from gadgets.rcl import MethodRunner


class Coordinator(threading.Thread):
    """
    The Coordinator keeps track of the state of all the gadgets
    and provides a few ways for gadgets to request information.
    Coordinator listens for the following events (this defines
    the zmq-based API for gadgets):

    register: When a gadget starts up it sends a register message.
              The register message just lets the Coordinator know
              that there is a new Gadget.  If the Gadget is a Device
              then the register message also includes the commands
              that turn the device on and off.

        >>> sockets.send('register', {
                'location': '<some location>',
                'name': '<some name>',
                'on': '<some on event>',
                'off': '<some off event>',    
            })
    
    method:  A method is a list of RCL messages.  If Coordinator
             receives a method message it starts up an instance
             of MethodRunner which then executes the method.

        >>> sockets.send('method', {
                'method': [
                    'turn on living room light',
                    'wait for 1 hour',
                    'turn off living room light'
                ]
            })

    update:  Whenever a Gadget reaches some new state, it should
             send an update message.  The Coordinator reads the message
             and updates its _state attribute.  The _state attribute
             is accessible to the outside world via the status message
             (see status below).  After Coordinator receives an update
             event, it sends an UPDATE event that contains the _state
             dict.  The UPDATE event is intended for GUIs.  Whenever
             something changes a GUI would want to be immediately
             notified.

         >>> sockets.send('update', {
                 '<some location>': {
                     '<some name>': {
                         'value': True
                    }
                }
            })

    status:  Send a status message and the response is the _state dict
             of the Coordinator.  Do with that state what you will.  The
             message should contain the id of the entity that sent the
             request.

        >>> sockets.send('status', {
                'id': '<some unique id>'
            })
        >>> _id, status = sockets.recv()

    events:  This message is also a request.  The response is all the
             on and off events that the Coordinator knows of due to
             gadgets registering and including 'on' and 'off' in the
             register message (see register above).  This is useful
             for GUIs to be able to know how to turn things on and
             off.

        >>> sockets.send('events', {
                'id': '<some unique id>'
            })
        >>> _id, events = sockets.recv()

    locations:  Another request that is useful for GUIs.  The response
                is a list of all the locations in the system that are
                registered with the Coordinator.

        >>> sockets.send('locations', {
                'id': '<some unique id>'
            })
        >>> _id, locations = sockets.recv()

    shutdown:  If this message is sent than all Gadget subclasses
               and the Coordinator will exit their main loops and
               die.  This message shuts down the whole system.

        >>> sockets.send('shutdown')

    error:  Any gadget can send this message and it will be stored
            in the _state attribute of the Coordinator.

        >>> sockets.send('error', {
                'error': '<some error message>'
                'id': '<some unique id>'
            })
    """

    def __init__(self, addresses):
        self._addresses = addresses
        self._ids = []

        self._sockets = None
        self._external_events = defaultdict(dict)
        self._event_handlers = {
            'register': self._handle_register,
            'method': self._run_method,
            'update': self._update,
            'status': self._status,
            'events': self._get_events,
            'locations': self._get_locations,
            'shutdown': self._shutdown,
            'error': self._get_error
            }
        
        self._state = {
            'locations': defaultdict(dict),
            'method': {},
            'errors': []
        }
        super(Coordinator, self).__init__()


    def run(self):
        stop = False
        while stop != True:
            stop = self._recv()
        self.sockets.close()

    @property
    def _events(self):
        return self._event_handlers.keys()

    @property
    def sockets(self):
        if self._sockets is None:
            self._sockets = Sockets(self._addresses, events=self._events)
        return self._sockets

    def _recv(self):
        event, message = self.sockets.recv()
        return_value = False
        if event in self._event_handlers:
            f = self._event_handlers[event]
            return_value = f(message)
        return return_value

    def _run_method(self, message):
        """
        a method message looks like:
        {'method': [
             'turn on light',
             'wait 10 seconds',
             'turn off light'
             ]
        }
        """
        self._state['method']['method'] = message['method']
        self._method_runner = MethodRunner(message['method'], self._addresses)
        self._method_runner.start()

    def _update_method(self, message):
        if 'step' in message['method']:
            self._state['method']['step'] = message['method']['step']
        elif 'complete' in message['method']:
            self._method_runner = None
            self._state['method'] = {}
        elif 'countdown' in message['method']:
            self._state['method']['countdown'] = message['method']['countdown']

    def _update_location(self, message):
        for location in message:
            for name, val in message[location].iteritems():
                value = val['value']
                units = val.get('units')
                self._do_update(location, name, value, units)

    def _update(self, message):
        """
        save the value in the message and pass it along to
        all the gadgets that care.
        
        To update, a gadget must send a message that looks
        like:

                  LOCATION         NAME 
        update {'livingroom': {'temperature': {'value': 33, 'units': 'C'}}}
        """
        if 'method' in message:
            self._update_method(message)
        else:
            self._update_location(message)
        self.sockets.send('UPDATE', self._state)

    def _do_update(self, location, name, value, units):
        self._state['locations'][location][name]['value'] = value
        self._state['locations'][location][name]['units'] = units
        
    def _is_valid_update_message(self, message):
        return True

    def _register(self, gadget):
        pass

    def _status(self, message):
        _id = message['id']
        self.sockets.send(str(_id), self._state)

    def _handle_register(self, message):
        """
        When a gadget starts up, it must register with the
        system.  Sensors do not have to register.  A register
        message looks like:

        register {
            'location': 'livingroom',
            'name': 'heater',
            'on': 'heat livingroom',
            'off': 'stop heating livingroom'
        }

        An id is assigned and a response is sent with a
        topic and message that looks like:

        livingroom heater registration {'id': 3}
        """
        if self._is_valid_registration_message(message):
            uid = message['uid']
            if uid not in self._ids:
                location = message['location']
                name = message['name']
                self._state['locations'][location][name] = {
                    'id': uid,
                    'value': False,
                }
                if message.get('on') and message.get('off'):
                    self._external_events[location][name] = {
                        'on': message['on'],
                        'off': message['off'],
                    }
                if message.get('input'):
                    self._state['locations'][location][name]['input'] = True
                self._ids.append(uid)
                
    def _is_valid_registration_message(self, message):
        return True
        #return set(message.keys()) == {'name', 'location', 'on', 'off', 'uid'}

    def _get_events(self, message):
        _id = message['id']
        self.sockets.send(str(_id), self._external_events)

    def _get_locations(self, message):
        _id = message['id']
        self.sockets.send(str(_id), self._state['locations'].keys())

    def _shutdown(self, message):
        return True

    def _get_error(self, message):
        self._state['errors'].append(message.get('error'))





class ADC(object):
    """
    ADC reads the adc pins (what else?).
    NOTE: On a beaglebone, you must do:
    
    # modprobe ti_tscadc
    for the adc pins to be enabled.
    """

    _base_path = '/sys/devices/platform/omap/tsc/{0}'

    def __init__(self, name):
        """
        name:  the name of the file that is read
        to obtain the adc value.
        """
        self._file_cache = None
        self._path = self._base_path.format(name)

    @property
    def value(self):
        value = int(self._file.read().replace('\x00', ''))
        self._file.seek(0)
        return value

    @property
    def _file(self):
        if self._file_cache is None:
            self._file_cache = open(self._path, 'r')
        return self._file_cache
            
    def close(self):
        self._file.close()




from gadgets.devices.device import Device
from gadgets.devices.heater.triggers.temperature import TemperatureTrigger
from gadgets.devices.heater.triggers.temperature import TemperatureTriggerFactory


class Cooler(Device):
    """
    Use this device to cool something down.
    """

    _units = ['celcius', 'C', 'fahrenheit', 'F']
    _on_template = 'cool {location}'
    _off_template = 'stop cooling {location}'

    def __init__(self, location, name, addresses, io_factory=None, trigger_factory=None, on=None, off=None):
        super(Cooler, self).__init__(location, name, addresses, io_factory, trigger_factory, on, off)
        self._target_temperature = None
        comparitor = lambda x, y: x <= y
        self._trigger_factory = TemperatureTriggerFactory(location, addresses, comparitor=comparitor)

    @property
    def events(self):
        return super(Cooler, self).events + ['update temperature']




import threading
import time
from gadgets import Sockets

class Trigger(threading.Thread):
    """
    A trigger is used to turn off a device in response to another event 
    (like a temperature reaching a certain value) or after waiting a certain
    amount of time.  When the Trigger is triggered it sends a 'completed' message
    to the rest of the system.
    """

    def __init__(self, location, on_event, off_event, message, addresses, target=None):
        self._location = location
        self._on_event = on_event
        self._off_event = off_event
        self._target = target
        self._message = message
        self._addresses = addresses
        self._sockets = None
        self._invalidated = False
        self._events = [off_event]
        super(Trigger, self).__init__()

    @property
    def sockets(self):
        if self._sockets is None:
            self._sockets = Sockets(self._addresses, events=self._events)
        return self._sockets

    def invalidate(self):
        pass

    def run(self):
        self.wait()
        if self._target is not None:
            self._target(self.status_message)
        event = 'completed {0}'.format(self._on_event)
        self.sockets.send(event)
        time.sleep(1)
        self.sockets.close()

    def wait(self):
        raise NotImplemented

    @property
    def status_message(self):
        """
        
        """
        return None
        
    def _get_units(self, message):
        pass

    



from gadgets.devices.valve.triggers.float import FloatTrigger, FloatTriggerFactory
from gadgets.devices.valve.triggers.gravity import GravityTrigger, GravityTriggerFactory
from gadgets.devices.valve.triggers.timer import TimerTrigger, TimerTriggerFactory
from gadgets.devices.valve.triggers.user import UserTrigger, UserTriggerFactory



import curses, time
from gadgets.ui.window import Window

class Confirm(Window):
    """ """

    def __init__(self, label):
        self._label = label
        self._width = len(label) + 2
        self._height = 4

    def __call__(self, screen):
        win = self._get_subwin(screen)
        self._add_title(win, self._label)
        win.addstr(2, 1, 'y  n'.center(len(self._label)))
        x = win.getch()
        return x == ord('y')


import re, json, os, copy, time, curses
from os import system
import curses

class MethodDisplay(object):
    
    def __init__(self, stdscr, x, y, data):
        self._name = 'method'
        self.row = 3
        self._win = stdscr.subwin(30, 50, y, x)
        self._steps = 26
        self._data = data
        self._method = data.get('method', {}).get('method', [])
        self._step = data.get('method', {}).get('step', 0)
        if len(self._method) > self._steps:
            self._max_start = len(self._method) - self._steps
        else:
            self._max_start = 0

    def _get_steps(self):
        if self._step == 0 or self._step == 1:
            start = 0
        elif self._step > self._max_start:
            start = self._max_start
        else:
            start = self._step - 1
        for i, j in enumerate(xrange(start, len(self._method))):
            if i < self._steps:
                yield i, j, self._method[j]

    def draw(self):
        self._win.clear()
        self._win.addstr(1, 2, self._name, curses.color_pair(1))
        for i, j, step in self._get_steps():
            if j == self._step:
                COLOR = curses.A_BOLD | curses.A_STANDOUT | curses.color_pair(1)
            else:
                COLOR = curses.color_pair(1)
            self._win.addstr(i + 2, 2, step, COLOR)
        y, x = self._win.getmaxyx()
        self._win.border()





from gadgets.devices.trigger import Trigger
from gadgets.io import Poller
from gadgets.errors import GadgetsError

class FloatTriggerFactory(object):

    def __init__(self, location, pin, volume, addresses):
        self._location = location
        self._pin = pin
        self._volume = volume
        self._addresses = addresses

    def __call__(self, on_event, off_event, message, target):
        return FloatTrigger(
            self._location,
            on_event,
            off_event,
            message,
            self._addresses,
            target,
            self._pin,
            self._volume
        )

class FloatTrigger(Trigger):
    """
    Waits until the gpio pin goes high, then calls the given target method.
    """

    _poller_class = Poller #this can be switched by tests

    def __init__(self, location, on_event, off_event, message, addresses, target, pin, volume):
        self._volume = volume
        self._pin = pin
        super(FloatTrigger, self).__init__(location, on_event, off_event, message, addresses, target=target)

    def wait(self):
        try:
            poller = self._poller_class(self._pin)
        except GadgetsError, e:
            raise GadgetsError(str(e), self.sockets)
        poller.wait()
        poller.close()

    @property
    def status_message(self):
        return {
            self._location: {
                'volume': {
                    'value': self._volume,
                    'units': 'liters'
                }
            }
        }

        
        


from gadgets.devices.trigger import Trigger
from gadgets.io.counter import Counter


class CounterTrigger(Trigger):

    def __init__(self, location, on_event, off_event, message, addresses, target=None, ticks=0, pin=None):
        self._ticks = ticks
        self._pin = pin
        super(CounterTrigger, self).__init__(location, on_event, off_event, message, addresses, target=target)

    def wait(self):
        counter = Counter(self._pin, self._ticks)
        counter.wait()

    


import time, threading, random
from nose.tools import eq_
from gadgets import Gadgets, Addresses, Sockets, Broker
from gadgets.devices import Switch, switch_factory

path = '/private/tmp/switch'


class FakeGPIO(object):

    def __init__(self):
        self.status = False


    def on(self):
        with open(path, 'w') as f:
            f.write('on')
        self.status = True

    def off(self):
        with open(path, 'w') as f:
            f.write('off')
        self.status = False

    def close(self):
        pass

def get_fake_gpio():
    return FakeGPIO()

class TestSwitch(object):

    def setup(self):
        port = random.randint(5000, 50000)
        self.addresses = Addresses(in_port=port, out_port=port + 1)
        self.sockets = Sockets(self.addresses, events=['update'])
        self.switch = Switch(
            'living room',
            'light',
            self.addresses,
            io_factory=get_fake_gpio,
        )
        self.gadgets = Gadgets([self.switch], self.addresses)
        
    def test_create(self):
        pass

    def test_on_and_off(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send('turn on living room light')
        print self.sockets.recv()
        with open(path, 'r') as f:
            eq_(f.read(), 'on')
        self.sockets.send('turn off living room light')
        print self.sockets.recv()
        time.sleep(0.2)
        with open(path, 'r') as f:
            eq_(f.read(), 'off')
        self.sockets.send('shutdown')

    def test_timed_on_and_off(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send('turn on living room light', {'units': 'seconds', 'value': 1})
        print self.sockets.recv()
        with open(path, 'r') as f:
            eq_(f.read(), 'on')
        print self.sockets.recv()
        with open(path, 'r') as f:
            eq_(f.read(), 'off')
        self.sockets.send('shutdown')

    def test_on_and_off_with_time_and_rcl(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send('turn on living room light for 0.5 seconds')
        print self.sockets.recv()
        with open(path, 'r') as f:
            eq_(f.read(), 'on')
        print self.sockets.recv()
        with open(path, 'r') as f:
            eq_(f.read(), 'off')
        self.sockets.send('shutdown')

    def test_switch_factory(self):
        switch = switch_factory('basement', 'fan', {'pin': None}, self.addresses)
        assert isinstance(switch, Switch)
        




import curses, time, datetime, json
from gadgets.ui.window import Window
from gadgets.ui.dialogs.prompt import Prompt


class Method(Window):
    """run a method"""

    _table = '''CREATE TABLE if not exists methods
             (id integer primary key, timestamp integer, title text, method text)'''

    _insert = '''INSERT INTO methods VALUES(null, ?, ?, ?)'''

    _delete = '''DELETE FROM methods WHERE id=?'''

    def __init__(self, socket, lock, db):
        self._socket = socket
        self._lock = lock
        db.ensure_table(self._table)
        self._db = db
        self._width = 40
        self._height = 18
        self._return_val = []
        self._dialogs = {
            'c': self._create_method,
            'r': self._run_method,
            'd': self._delete_method,
            }

    def _save(self, title, method):
        timestamp=datetime.datetime.now()
        self._db.connection.execute(self._insert, (int(timestamp.strftime('%s')), title, method))
        self._db.connection.commit()


    def _create_method(self, win):
        win.clear()
        win.border()
        self._add_title(win, 'enter method, end with 2 newlines')
        curses.echo()
        method = ''
        row = 2
        while not method.endswith('\n\n'):
            win.move(row ,1)
            method += win.getstr() + '\n'
            row += 1
        prompt = Prompt('method title', str)
        title = prompt(win)
        curses.noecho()
        if method != '':
            self._save(title, method)

    def _get_methods(self):
        cursor = self._db.connection.cursor()
        cursor.execute('SELECT * FROM methods')
        return dict([
            (str(m[0]), dict(
                id=m[0],
                title=m[2],
                method=m[3])) for m in cursor.fetchall()])

    def _print_methods(self, win, row, methods):
        for method in methods.itervalues():
            win.addstr(row, 1, '{0}: {1}'.format(method['id'], method['title']))
            row += 1
        return row

    def _run_method(self, win):
        win.clear()
        win.border()
        self._add_title(win, 'select a method')
        methods = self._get_methods()
        row = 2
        row = self._print_methods(win, row, methods)
        curses.echo()
        win.move(row, 1)
        _id = win.getstr()
        if _id != '':
            method = methods[_id]
            method = method['method'].split('\n')
            self._return_val = method
            self._socket.send('method', {'method': method})
        curses.noecho()

    def _delete_method(self, win):
        win.clear()
        win.border()
        self._add_title(win, 'select a method')
        methods = self._get_methods()
        row = 2
        row = self._print_methods(win, row, methods)
        win.move(row, 1)
        curses.echo()
        _id = win.getstr()
        if _id != '':
            self._db.connection.execute(self._delete, (_id))
            self._db.connection.commit()
        curses.noecho()

    def _ask_user(self, win):
        x = ''
        while x != 'x' and x != 'r':
            win.clear()
            self._add_title(win, self.__doc__)
            win.addstr(2, 1, 'c: create a method')
            win.addstr(3, 1, 'd: delete a method')
            win.addstr(4, 1, 'r: run a method')
            win.addstr(5, 1, 'x: exit')
            win.move(7,1)
            x = chr(win.getch())
            if x in self._dialogs:
                method = self._dialogs[x]
                method(win)

    def __call__(self, screen, data):
        self._lock.acquire()
        win = self._get_subwin(screen)
        self._ask_user(win)
        self._lock.release()
        return self._return_val
        



# -*- coding: utf-8 -*-
import re
import json



number_re = re.compile('[0-9-]*\.?[0-9]+')
command_re = re.compile('(?:^|(?<=\s))[0-9]*\.?[0-9]+(?=\s|$)')
    
def split_command(command, match):
    start = match.start()
    end = match.end()
    event, value, units = command[:start].strip(), command[start:end], command[end:].strip()
    if event.endswith(' for'):
        event = event[:-4]
    elif event.endswith(' to'):
        event = event[:-3]
    return event, float(value), units

def parse_command(command, message=None):
    command = command.strip()
    match = number_re.search(command)
    if match:
        event, value, units = split_command(command, match)
        message = {'value': value, 'units': units}
    else:
        event = command
        message = message
    return str(event), message



from gadgets.devices.device import Device

class Button(Device):

    def run(self):
        self.io.start()
        super(Button, self).run()

    @property
    def events(self):
        """
        """
        return []

    def get_on_event(self, on=None):
        return None

    def get_off_event(self, on=None):
        return None

    def _register(self):
        self.sockets.send(
            'register',
            {
                'location': self._location,
                'name': self._name,
                'uid': self.uid,
                'input': True
            }
        )

    


import time
import threading
from gadgets.io import GPIOFactory

class PWMFactory(object):

    def __init__(self, pin):
        self._pin = pin

    def __call__(self):
        pwm = PWM(GPIOFactory(self._pin, direction='out'))
        pwm.start()
        return pwm

class PWM(threading.Thread):
    """
    I started off with a beaglebone PWM pin to control
    the amount of heating done by a electric heating
    coil, but that might be overkill.  I've been reading
    that electric stoves have a duty cycle of around 10 
    seconds, and that can be handled by gpio.
    """

    def __init__(self, gpio_factory):
        self._gpio_factory = gpio_factory
        self._gpio = None
        self._duty_cycle = 4.0
        self._on_time = 0.0
        self._off_time = 4.0
        self._stop_event = threading.Event()
        self.status = False
        self._lock = threading.RLock()

    def run(self):
        while not self._stop_event.is_set():
            self._lock.acquire()
            if self._on_time > 0:
                if not self.gpio.status:
                    self.gpio.on()
                time.sleep(self._on_time)
            if self._off_time > 0:
                if self.gpio.status:
                    self.gpio.off()
                time.sleep(self._off_time)
            self._lock.release()

    @property
    def gpio(self):
        if self._gpio is None:
            self._gpio = self._gpio_factory()
        return self._gpio

    def on(self):
        """
        turns on the pwm and sets the duty cycle to 100
        """
        self.duty_percent = 100
        self.status = True
        self.gpio.on()

    def off(self):
        """
        turns off the pwm and sets the duty cycle to 0
        """
        self.duty_percent = 0
        self.status = False
        self.gpio.off()

    def close(self):
        self._lock.acquire()
        self._stop_event.set()
        self.gpio.close()
        self._lock.release()

    @property
    def duty_percent(self):
        if self._off_time == 0.0:
            val = 100
        else:
            val = int((self._on_time / self._off_time) / 100.0)
        return val

    @property
    def value(self):
        return self.duty_percent

    @duty_percent.setter
    def duty_percent(self, value):
        """
        duty_percent(value)
        value: an integer from 0 to 100

        Writes to the sysfs duty_percent interface.  If the pwm
        was turned off before this call, the pwm will be turned
        on.
        """
        self._lock.acquire()
        self._on_time = self._duty_cycle * (float(value) / 100.0)
        self._off_time = self._duty_cycle - self._on_time
        self._lock.release()
        


import time, random, threading
from nose.tools import eq_
from gadgets.devices.valve.triggers import GravityTriggerFactory
from gadgets import Addresses, Gadgets, Sockets

class TestGravityTrigger(object):

    def setup(self):
        self._off = False
        port = random.randint(5000, 50000)
        self.addresses = Addresses(in_port=port, out_port=port + 1)
        self.sockets = Sockets(self.addresses)
        self.gadgets = Gadgets([], self.addresses)
        
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.gadgets.coordinator._state['locations']['barrel'] = {'volume': {'value': 3, 'units': 'liters'}}
        self.gadgets.coordinator._state['locations']['bucket'] = {'volume': {'value': 1, 'units': 'liters'}}
        factory = GravityTriggerFactory(
            'bucket',
            self.addresses,
            'barrel',
            20.0,
            1.0,
            1.0
        )
        self.trigger = factory(
            'fill bucket',
            'stop_filling_bucket',
            {'units':'liters', 'value': 2.0},
            self.off
            )
        

    def teardown(self):
        self.sockets.close()

    def off(self, message):
        self._off = True

    def _test_create(self):
        self.sockets.send('shutdown')
        time.sleep(0.2)

    def test_run(self):
        self.trigger.start()
        while not self._off:
            time.sleep(0.1)
        self.sockets.send('shutdown')
        eq_(self._off, True)
        #eq_(self.gadgets._state['locations'], {'barrel': {'volume': {'units': 'liters', 'value': 2}}, 'bucket': {'volume': {'units': 'liters', 'value': 2}}})



import time, threading, random
from nose.tools import eq_
from gadgets import Addresses, Sockets, Broker

class TestSockets(object):

    def test_broker(self):
        p = random.randint(3000, 50000)
        addresses = Addresses(in_port=p, out_port=p+1)
        broker = Broker(addresses)
        sockets = Sockets(addresses)
        broker.start()
        time.sleep(0.5)
        sockets.send('shutdown', {})
        time.sleep(1)
        

    



from gadgets.devices.switch.triggers.switch_timer import SwitchTimer
from gadgets.devices.device import Device
        
class Switch(Device):
    """
    A Switch is used to control items that just need to
    be turned on and off.  Here is an example:
    
    >>> from gadgets.pins.beaglebone import pins
    >>> from gadgets import get_gadgets
    >>> arguments = {
    ...     "locations": {
    ...         "living room": {
    ...             "light": {
    ...             "type": "switch",
    ...             "pin": pins["gpio"][8][3]
    ...             },
    ...         "back yard": {
    ...             "sprinklers": {
    ...             "type": "switch",
    ...             "pin": pins["gpio"][8][11],
    ...             },
    ...         },
    ...     },
    ... }
    >>> gadgets = get_gadgets(arguments)
    >>> gadgets.start()

    Now you can turn it on by creating a socket and giving the switches
    a RCL command::

    >>> from gadgets import Sockets
    >>> s = Sockets()
    >>> s.send("turn on living room light")

    To turn on the sprinklers in the back yard for 15 minutes, you would send
    this command:

    >>> s.send("turn on back yard sprinklers for 15 minutes")
    """
    
    _units = ['minutes', 'minute', 'seconds', 'second', 'hours', 'hour']
    _trigger_factory = SwitchTimer

    def __init__(self, location, name, addresses, io_factory=None, trigger_factory=None, on=None, off=None, momentary=False):
        self._momentary = momentary
        super(Switch, self).__init__(location, name, addresses, io_factory=io_factory, trigger_factory=trigger_factory, on=on, off=off)

    def _get_trigger(self, message):
        return self._trigger_factory(self._location, self._on_event, self._off_event, message, self._addresses, target=self.off)

    def on(self, message=None):
        if self._momentary:
            message = {'units': 'seconds', 'value': 0.1}
        super(Switch, self).on(message)



import threading, uuid, datetime, time, zmq

from gadgets.devices.trigger import Trigger
from gadgets import Sockets
from gadgets.devices.valve.triggers.volume_converter import VolumeConverter


class GravityTriggerFactory(object):

    def __init__(self, location, addresses, source, tank_radius, valve_radius, valve_coefficient):
        """
        location:          the location in the gadgets system (room, backyard, fish tank, etc.)
        addresses:    the gadgets.Options object for the system
        source:            the container that supplies this location
        tank_radius:       radius of the container
        valve_radius:      duh
        valve_coefficient: a number that can be tweaked to get the calculations
                           correct.  try starting with 0.8
        """
        self._location = location
        self._addresses = addresses
        self._source = source
        self._tank_radius = tank_radius
        self._valve_radius = valve_radius
        self._valve_coefficient = valve_coefficient
        

    def __call__(self, on_event, off_event, message, target):
        uid = str(uuid.uuid1())
        target_volume = message.get('value')
        sockets = Sockets(self._addresses, events=[str(uid)])
        sockets.send('status', {'id': uid})
        returned_uid, message = sockets.recv()
        source_volume = message['locations'][self._source].get('volume', {}).get('value', 0.0)
        starting_volume = message['locations'][self._location].get('volume', {}).get('value', 0.0)
        volume_converter = VolumeConverter(
            tank_radius=self._tank_radius,
            valve_radius=self._valve_radius,
            coefficient=self._valve_coefficient
        )
        volume_args = dict(
            source=self._source,
            target_volume=target_volume,
            starting_volume=starting_volume,
            source_volume=source_volume,
            volume_converter=volume_converter
            )
        
        return GravityTrigger(
            self._location,
            on_event,
            off_event,
            message,
            self._addresses,
            target,
            volume_args
        )

class Updater(threading.Thread):

    def __init__(self, start_time, update):
        self._start_time = start_time
        self._update = update
        self._stop_event = threading.Event()
        super(Updater, self).__init__()

    def run(self):
        while not self._stop_event.is_set():
            time_difference = datetime.datetime.now() - self._start_time
            self._update(time_difference.total_seconds())
            time.sleep(1)

    def stop(self):
        self._stop_event.set()

class GravityTrigger(Trigger):

    def __init__(self, location, on_event, off_event, message, addresses, target, volume_args):
        """
        
        """
        self._location = location
        self._target_volume = volume_args['target_volume']
        self._starting_volume = volume_args['starting_volume']
        self._source_volume = volume_args['source_volume']
        self._source = volume_args['source']
        self._volume_converter = volume_args['volume_converter']
        self._invalidated = False
        self._updater = None
        super(GravityTrigger, self).__init__(location, on_event, off_event, message, addresses, target=target)

    def wait(self):
        self._start_time = datetime.datetime.now()
        if self._target_volume is None:
            self._wait_for_stop_event()
        else:
            self._wait_for_time()

    def _wait_for_stop_event(self):
        poller = zmq.Poller()
        poller.register(self.sockets.subscriber, zmq.POLLIN)
        run = True
        while run:
            socks = dict(poller.poll(1000))
            now = datetime.datetime.now()
            if self.sockets.subscriber in socks and socks[self.sockets.subscriber] == zmq.POLLIN:
                event, message = self.sockets.recv()
                if event == self._off_event:
                    run = False
            time_difference = now - self._start_time
            self._update(time_difference.total_seconds())

    def _wait_for_time(self):
        volume = self._target_volume - self._starting_volume
        if volume <= 0:
            self._final_volume = self._starting_volume
            self._final_source_volume = self._source_volume
        else:
            drain_time = self._volume_converter.get_drain_time(self._source_volume, volume)
            self._updater = Updater(self._start_time, self._update)
            self._updater.start()
            time.sleep(drain_time)
            self._updater.stop()
            if not self._invalidated:
                self._final_volume = self._target_volume
                self._final_source_volume = self._source_volume - volume
            else:
                self._final_volume = None
                self._final_source_volume = None

    def _update(self, difference):
        volume = self._volume_converter.get_volume(self._source_volume, difference)
        source_volume = self._source_volume - volume
        total_volume = self._starting_volume + volume
        msg = self._get_message(total_volume, source_volume)
        self.sockets.send('update', msg)

    def _get_message(self, my_volume, source_volume):
        return {
            self._location: {
                'volume': {
                    'value': my_volume,
                    'units': 'liters'
                }
            },
            self._source: {
                'volume': {
                    'value': source_volume,
                    'units': 'liters'
                }
            }
        }

    def invalidate(self):
        if self._updater is not None:
            self._updater.stop()

    @property
    def status_message(self):
        if self._final_volume is not None:
            msg = self._get_message(self._final_volume, self._final_source_volume)
        else:
            msg = None
        return msg



from gadgets.devices.button.io import ButtonPollerFactory
from gadgets.devices.button.button import Button


def button_factory(location, name, arguments, addresses, io_factory=None):
    if io_factory is None:
        io_factory = ButtonPollerFactory(location, name, addresses, arguments['pin'])
    return Button(
        location=location,
        name=name,
        addresses=addresses,
        io_factory=io_factory,
    )


import curses

class Window(object):

    def _get_ch(self, win):
        x = win.getch()
        if x > 0 and x < 256:
            x = chr(x)
        else:
            x = ''
        return x

    def _get_center(self, screen):
        height, width = screen.getmaxyx()
        col = (width/2) - (self._width/2)
        row = (height/2) - (self._height/2)
        return row, col
    
    def _get_subwin(self, screen):
        row, col = self._get_center(screen)
        win = screen.derwin(self._height, self._width, row, col)
        win.clear()
        win.border()
        return win

    def _add_title(self, win, title):
        win.addstr(1, 1, '{0}'.format(title).center(self._width - 2), curses.A_BOLD | curses.A_STANDOUT | curses.color_pair(1))




from gadgets.devices.motor.triggers.counter import CounterTrigger

class MotorTriggerFactory(object):

    def __init__(self, location, addresses, units, pin):
        self._units = units
        self._pin = pin
        self._location = location
        self._addresses = addresses

    def __call__(self, on_event, off_event, message, target):
        print message
        units = message.get('units')
        if units in self._units['all']:
            if units in self._units['counter']:
                ticks = message.get('value', 0)
                return CounterTrigger(
                    self._location,
                    on_event,
                    off_event,
                    message,
                    self._addresses,
                    target,
                    abs(ticks),
                    self._pin
                )


import time, threading, random, multiprocessing
from nose.tools import eq_
from gadgets import Gadgets, Addresses, Sockets, Broker
from gadgets.devices import cooler_factory

path = '/private/tmp/cooler'
port = random.randint(3000, 50000)


class FakeGPIO(object):

    def __init__(self):
        print 'gpio init'
        self.addresses = Addresses(in_port=port, out_port=port+1)
        self.sockets = Sockets(self.addresses)
        self.status = False

    def _write(self, msg):
        print 'gpio write', msg
        with open(path, 'w') as f:
            f.write(msg)
        print 'gpio send'
        self.sockets.send('test update')

    def on(self):
        print 'gpio on'
        self._write('on')
        self.status = True

    def off(self):
        self._write('off')
        self.status = False

    def close(self):
        pass
        

def get_fake_gpio(*args, **kw):
    return FakeGPIO()

class TestCooler(object):

    def setup(self):
        self.addresses = Addresses(in_port=port, out_port=port + 1)
        self.sockets = Sockets(self.addresses, events=['test update'])
        self.cooler = cooler_factory('tank', 'cooler', {}, self.addresses, io_factory=get_fake_gpio)
        self.gadgets = Gadgets([self.cooler], self.addresses)

    def teardown(self):
        self.sockets.send('shutdown')
        time.sleep(0.2)
        self.sockets.close()

    def test_create(self):
        pass

    def _read(self):
        with open(path, 'r') as f:
            return f.read()

    def test_on_and_off(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send('cool tank', {'units': 'C', 'value': 20})
        self.sockets.recv() #sync
        eq_(self._read(), 'on')
        time.sleep(0.2)
        self.sockets.send('update temperature', {'tank': {'temperature': {'value': 20, 'units': 'C'}}})
        self.sockets.recv() #sync
        eq_(self._read(), 'off')
        time.sleep(0.2)
        

        



import time
from gadgets.devices.device import Device

class Motor(Device):
    """
    Motor is a general class for controlling a motor.  It should be
    able to control any motor, as long as it is initialized with
    an io object that can interface with the physical interface
    that is being used.
    

    >>> from gadgets import get_gadgets
    >>> from gadgets.pins.beaglebone import pins
    >>> brewery_def = {
    ...     'locations': {
    ...         'front yard': {
    ...             'camera': {
    ...                 'type': 'motor',
    ...                 'gpio_a': 'gpio_a': pins['gpio'][8][24],
    ...                 'gpio_b': pins['gpio'][8][25],
    ...                 'pwm': pins['pwm'][8][13],
    ...                 'poller': pins['gpio'][8][27]
    ...            }
    ...         }
    ...     }
    ... }

    If the motor you are controlling comes with an encoder (the example
    above assumes an encoder) then you can turn it on with a RCL
    command that uses 'ticks' as its units::

    >>> from gadgets import Sockets
    >>> sock = Sockets()
    >>> sock.send('turn on top motor for 40 ticks')
    
    """

    _time_units= ('hours', 'minutes', 'seconds', 'hour', 'minute', 'second')
    _percent_units = ('percent',)
    _counter_units = ('ticks',)
    _units = _time_units + _percent_units + _counter_units

    def _should_get_trigger(self, message):
        units = message.get('units')
        return units in self._time_units or units in self._counter_units

    def on(self, message):
        """
        value should range from -100 to 100 for full power counter-
        clockwise to full power clockwise.
        """
        if self._should_get_trigger(message):
            self._invalidate_trigger()
            self._off_trigger = self._get_trigger(message)
            self._off_trigger.start()
        units = message.get('units')
        if units == 'ticks':
            ticks = message['value']
            if ticks < 0:
                value = -100
            else:
                value = 100
            time.sleep(0.1)
            self.io.on({'value': value, 'units': 'percent'})
        else:
            self.io.on(message)
        self._update_status(True)

    def off(self, message):
        """
        value should range from -100 to 100 for full power counter-
        clockwise to full power clockwise.
        """
        self.io.off()
        self._update_status(False)
        
    


from gadgets.io.io import IO
from gadgets import Sockets


class ShiftRegisterIOFactory(object):
    
    def __init__(self, addresses, channel):
        self._addresses = addresses
        self._channel = channel
    
    def __call__(self):
        return ShiftRegisterIO(self._addresses, self._channel)


class ShiftRegisterIO(IO):

    """
    Implements the gadgets io api (on, off, close, and status)
    see gadgets.io.io.

    A Beaglebone has lots of gpio pins available, so why use a
    power logic shift register (such as a TPIC6B595)?  The gpio
    pins cannot provide enough current to power much more than
    a transistor.  So if you want to turn on a sollid state
    relay or a solenoid then you need to use a transistror
    or mosfet.  The use of the shift register elminates the
    need for transistors in many cases.  Also, you can chain
    multiple shift registers together and get many more gpio
    pins than you could get with a single beaglebone:
    
    >>> from gadgets import get_gadgets
    >>> arguments = {
    ...     "locations": {
    ...         "back yard": {
    ...             "sprinklers": {
    ...                 "type": "shift register switch",
    ...                 "channel": 3,
    ...                 "on": "water {location}",
    ...                 "off": "stop watering {location}"
    ...            }
    ...         }
    ...     }
    ... }
    >>> gadgets = get_gadgets(arguments)

    Note that you can pass in an 'on' and 'off' argument for
    a device.  This allows you to turn the device on with a
    command other than the default.  The default on and off
    commands in this case would have been 'turn on back yard
    sprinklers' and 'turn off back yard sprinklers'.  Now in
    another terminal:

    >>> from gadgets import Sockets
    >>> sock = Sockets()
    >>> sock.send('water back yard for 10 minutes')
    """

    def __init__(self, addresses, channel):
        self._sockets = Sockets(addresses)
        self._channel = channel
        self._status = False

    @property
    def status(self):
        return self._status
        
    def on(self):
        self._send(True)

    def off(self):
        self._send(False)

    def close(self):
        self._sockets.close()

    @property
    def value(self):
        return self.status

    def _send(self, value):
        self._sockets.send(
            'shift register server',
            {'value': value, 'channel': self._channel}
        )
        self._status = value



import time

class GadgetsError(Exception):
    """
    raises an Exception, and if initialized with
    a sockets object, will send the error message
    to the whole gadgets system.
    """

    def __init__(self, msg, sockets=None, uid=None):
        if sockets is not None:
            sockets.send('error', {'error': msg, 'id': uid})
            time.sleep(0.2) #give the socket time before the error is raised
        super(GadgetsError, self).__init__(msg)
        
        


import time
from nose.tools import eq_
from gadgets.devices.heater.triggers.temperature import TemperatureTrigger
from gadgets import Addresses, Sockets, Broker

class TestTemperatureTrigger(object):

    def setup(self):
        self.addresses = Addresses()
        comparitor = lambda x, y: x >= y
        self.trigger = TemperatureTrigger(
            'tank',
            'heat tank',
            'stop heating tank',
            {'units': 'C', 'value': 88},
            self.addresses,
            comparitor
        )

    def test_create(self):
        eq_(self.trigger._target_temperature, 88)

    def test_run(self):
        b = Broker(self.addresses)
        b.start()
        time.sleep(1)
        self.trigger.start()
        sockets = Sockets(self.addresses, ['completed heat tank'])
        sockets.send('update temperature', {'tank': {'temperature': {'value': 87, 'units': 'C'}}})
        time.sleep(0.1)
        sockets.send('update temperature', {'tank': {'temperature': {'value': 88, 'units': 'C'}}})
        time.sleep(0.1)
        event, message = sockets.recv()
        eq_(event, 'completed heat tank')
        sockets.send('shutdown')
        sockets.close()



from gadgets.devices.device import Device
from gadgets.errors import GadgetsError

class Valve(Device):
    """
    Valve is pretty close to Switch, but uses volume units
    for its triggers instead of time units.  On the surface
    Valve is pretty simple, but it has 4 different triggers 
    making it the most complicated Device.  All of these
    triggers were designed for my automatic brewery.  They are::

    FloatTrigger -   Waits until a float switch is activated.
    
    GravityTrigger - Uses the equation of water flowing through
                     an orfice to determine how much water has
                     flowed in a given amount of time.
    
    TimerTrigger -   The user initializes it with the maximum
                     amount of time required for the source tank
                     to completely drain.  This is useful for a
                     tank that is full of grains, making the equation
                     used by GravityTrigger inaccurate.
    
    UserTrigger -    Sends a message to whatever user-inteface is
                     being used.  The user-interface should display
                     a message asking the user to indicate when the
                     source tank is empty.  This is useful when the
                     valve is really a pump.  The user (via the user-
                     interface) confirms the tank is empty and the pump
                     shuts off immediately.  Most pumps don't like to
                     be run dry.
    
    The valve triggers have more documentation.

    Here is an example with four tanks (to illustrate all four triggers).
    The first tank uses a FloatTrigger and the one below it uses a GravityTrigger.

    >>> from gadgets import get_gadgets
    >>> from gadgets.pins.beaglebone import pins
    >>> brewery_def = {
    ...     'locations': {
    ...         'hot liquor tank': {
    ...             'valve': {
    ...                 'type': 'valve',
    ...                 'pin': pins['gpio'][8][11],
    ...                 'trigger': {
    ...                     'type': 'float',
    ...                     'pin': pins['gpio'][8][12],
    ...                     'volume': 26.5,
    ...                 }
    ...             }
    ...         },
    ...         'mash tun': {
    ...             'valve': {
    ...                 'type': 'valve',
    ...                 'pin': pins['gpio'][8][12],
    ...                 'trigger': {
    ...                     'type': 'gravity',
    ...                         'source': 'hot liquor tank',
    ...                         'tank_radius': 7.5,
    ...                         'valve_radius': 0.1875,
    ...                         'valve_coefficient': 0.43244,    
    ...                 }
    ...             }
    ...         },
    ...         'boiler': {
    ...             'valve': {
    ...                 'type': 'valve',
    ...                 'pin': pins['gpio'][8][12],
    ...                 'trigger': {
    ...                     'type': 'timer',
    ...                         'source': 'mash tun',
    ...                         'drain_time': 240 #seconds
    ...                 }
    ...             }
    ...         }
    ...         'carboy': {
    ...             'valve': {
    ...                 'type': 'valve',
    ...                 'pin': pins['gpio'][8][12],
    ...                 'trigger': {
    ...                     'type': 'user',
    ...                         'source': 'boiler',
    ...                 }
    ...             }
    ...         },
    ...     }
    ... }
    >>> from gadgets import Sockets
    >>> sock = Sockets()
    >>> sock.send('fill hot liquor tank')

    You don't have to tell the hot liquor tank a volume in the command,
    because it can only be filled to one known volume (26.5 liters in this
    case)

    Once the hot liquor tank is filled you can add any amount of water into the
    mash tun:

    >>> sock.send('fill mash tun to 6.5 liters')

    Knowing about the laws of physics, the GravityTrigger will turn off its valve
    after the appropriate amount of time.
    """

    _units = ['liters', 'gallons']
    _on_template = 'fill {location}'
    _off_template = 'stop filling {location}'

    def _get_trigger(self, message):
        try:
            trigger = self._trigger_factory(
                self._on_event,
                self._off_event,
                message,
                self.off
            )
        except GadgetsError, e:
            self.off()
        else:
            return trigger

    def _should_get_trigger(self, message):
        return True #we don't want tanks overflowing

    def _register(self):
        super(Valve, self)._register()
        self.sockets.send('update', {self._location: {'volume': {'value': 0.0, 'units': 'liters'}}})




import os
from gadgets.io.io import IO

class PWMFactory(object):

    def __init__(self, pin, frequency=20000, mux_path='/sys/kernel/debug/omap_mux'):
        self._pin = pin
        self._frequency = frequency
        self._mux_path = mux_path

    def __call__(self):
        return PWM(self._pin, self._frequency, self._mux_path)
        

class PWM(IO):
    """
    Uses the pwm pins' sysfs interface.  You can use a pwm pin like this:

        >>> from gadgets.pins.beaglebone import pins
        >>> from gadgets.io import GPIO
        >>> pwm = PWM(pins['pwm'][8][13])

    Now you can turn it on and vary the duty percent:

        >>> pwm.on() #turns the pin on with a 100 duty_percent
        >>> pwm.duty_percent = 50
    
    NOTE:  The beaglebone pwm clock must be enabled before using this class.
    You can enable it by calling enable-pwm from the command line (requires
    gadgets and python-mmap to be installed)
    """
    
    _mux = 'echo {0} > {1}/{2}'
    _command = 'echo {0} > /sys/class/pwm/{1}/{2}'

    def __init__(self, pin, frequency=20000, mux_path='/sys/kernel/debug/omap_mux'):
        """
        mux:  the name that appears in the omap_mux directory
        directory: the directory for the pwm pin in /sys/class/pwm
        mode:  the mux mode of the pin that enables pwm
        frequency:  the pwm frequency in Hz
        mux_path:  defaults to /sys/kernel/debug/omap_mux
        """
        mux = pin['mux']
        directory = pin['directory']
        mode = pin['mode']
        self._current_value = None
        self._directory = directory
        os.system(self._mux.format(mode, mux_path, mux))
        self._write_value('run', '0')
        self._write_value('duty_percent', '0')
        self._write_value('period_freq', str(frequency))
        self._status = False

    @property
    def status(self):
        return self._status

    def on(self):
        """
        turns on the pwm and sets the duty cycle to 100
        """
        self.duty_percent = 100
        self._status = True

    def off(self):
        """
        turns off the pwm and sets the duty cycle to 0
        """
        self.duty_percent = 0
        self._status = False
        self._write_value('run', '0')

    def close(self):
        pass

    @property
    def duty_percent(self):
        return self._current_value

    @property
    def value(self):
        return self.duty_percent

    @duty_percent.setter
    def duty_percent(self, value):
        """
        duty_percent(value)
        value: an integer from 0 to 100

        Writes to the sysfs duty_percent interface.  If the pwm
        was turned off before this call, the pwm will be turned
        on.
        """
        if self._current_value != value:
            self._current_value = value
            self._write_value('run', '0')
            self._write_value('duty_percent', value)
            self._write_value('run', '1')

    def _write_value(self, path, value):
        command = self._command.format(value, self._directory, path)
        os.system(command)



import curses, time
from gadgets.ui.window import Window

class Prompt(Window):
    """ """

    def __init__(self, label, cast):
        self._label = label
        self._cast = cast
        self._width = 40
        self._height = 4

    def __call__(self, screen):
        win = self._get_subwin(screen)
        self._add_title(win, self._label)
        win.move(2,1)
        curses.echo()
        curses.curs_set(1)
        x = win.getstr()
        curses.curs_set(0)
        curses.noecho()
        try:
            x = self._cast(x)
        except ValueError:
            x = None
        return x
        


from gadgets.devices.cooler.cooler_factory import cooler_factory
from gadgets.devices.heater.electric_heater_factory import electric_heater_factory
from gadgets.devices.motor.motor_factory import motor_factory
from gadgets.devices.switch.gpio.factory import switch_factory
from gadgets.devices.switch.xbee.factory import xbee_factory
from gadgets.devices.valve.valve_factory import ValveFactory
from gadgets.devices.cooler.cooler_factory import cooler_factory

from gadgets.devices.switch.switch import Switch
from gadgets.devices.valve.valve import Valve
from gadgets.devices.heater.electric_heater import ElectricHeater



import curses, time
from gadgets.ui.window import Window
from gadgets.ui.dialogs.prompt import Prompt

class Command(Window):
    """enter a command"""

    def __init__(self, parent, lock):
        self._parent = parent
        self._lock = lock

    def __call__(self, screen, data):
        with self._lock:
            p = Prompt('enter a command', str)
            command = p(screen)
            self._parent.sockets.send(command)



from setuptools import setup, find_packages
import sys, os

version = '0.1.1'

long_description = ''

with open('README.txt') as buf:
    long_description += buf.read()

#with open('CHANGES.txt') as buf:
    #long_description += buf.read()

    
setup(name='gadgets',
      version=version,
      description="A physical computing framework",
      long_description=long_description,
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='beaglebone physical computing',
      author='Craig Swank',
      author_email='craigswank@gmail.com',
      url='https://bitbucket.org/cswank/gadgets',
      license='MIT',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'pyzmq',
      ],
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      enable-pwm=gadgets.utils.enable_pwm:main
      gadgets=gadgets.ui.ui:main
      """,
      )



import time, uuid
from gadgets.devices.trigger import Trigger
from gadgets import Sockets

class TimerTriggerFactory(object):

    def __init__(self, location, addresses, source, drain_time=None):
        self._location = location
        self._source = source
        if drain_time is not None:
            self._drain_time = drain_time * 60
        else:
            self._drain_time = None
        self._addresses = addresses

    def __call__(self, on_event, off_event, message, target):
        uid = str(uuid.uuid1())
        sockets = Sockets(self._addresses, events=[uid])
        sockets.send('status', {'id': uid})
        returned_uid, message = sockets.recv()
        source_volume = message['locations'][self._source].get('volume', {}).get('value', 0.0)
        volume = message['locations'][self._location].get('volume', {}).get('value', 0.0)
        sockets.close()
        return TimerTrigger(
            self._location,
            on_event,
            off_event,
            message,
            self._addresses,
            target,
            volume,
            self._source,
            source_volume,
            self._drain_time
        )

class TimerTrigger(Trigger):

    def __init__(self, location, on_event, off_event, message, addresses, target, volume, source, source_volume, drain_time=None):
        self._source = source
        self._source_volume = source_volume
        self._volume = volume
        self._drain_time = drain_time
        self._invalidated = False
        super(TimerTrigger, self).__init__(location, on_event, off_event, message, addresses, target=target)

    def wait(self):
        time.sleep(self._drain_time)

    def invalidate(self):
        self._invalidated = True

    @property
    def status_message(self):
        if not self._invalidated:
            msg = {
                self._location: {
                    'volume': {
                        'value': self._source_volume + self._volume,
                        'units': 'liters'
                    }
                },
                self._source: {
                    'volume': {
                        'value': 0,
                        'units': 'liters'
                    }
                }
            }
            return msg



from gadgets.io import GPIOFactory
from gadgets.io import PWMFactory
from gadgets.devices.motor.triggers import MotorTriggerFactory
from gadgets.devices.motor.motor_io import MotorIOFactory
from gadgets.devices.motor.motor import Motor


def motor_factory(location, name, arguments, addresses):
    gpio_a = GPIOFactory(arguments['gpio_a'], direction='out')
    gpio_b = GPIOFactory(arguments['gpio_b'], direction='out')
    pwm = PWMFactory(arguments['pwm'], frequency=20000)
    io_factory = MotorIOFactory(gpio_a, gpio_b, pwm)
    units = {
        'all': Motor._units,
        'time': Motor._time_units,
        'percent': Motor._percent_units,
        'counter': Motor._counter_units
    }
    trigger_factory = MotorTriggerFactory(
        location,
        addresses,
        units,
        arguments['poller']
    )
        
    return Motor(
        location=location,
        name=name,
        addresses=addresses,
        io_factory=io_factory,
        trigger_factory=trigger_factory,
        on=arguments.get('on'),
        off=arguments.get('off'),
    )
    


import time, threading

from gadgets import Gadget
from gadgets.rcl.command_parser import parse_command

class Countdown(threading.Thread):
    
    def __init__(self, seconds, target):
        self._seconds = seconds
        self._target = target
        super(Countdown, self).__init__()

    def run(self):
        t = 0
        while t < self._seconds:
            time.sleep(1)
            t += 1
            self._target(self._seconds - t)
        self._target(0)

class MethodRunner(Gadget):
    """
    reads a list of RCL messages and uses the
    command parser to turn them into Gadget
    messages for the system.
    """

    
    _time_units = ['seconds', 'minutes', 'hours', 'second', 'minute', 'hour']

    def __init__(self, method, addresses):
        super(MethodRunner, self).__init__('method', 'runner', addresses)
        self._method = method
        self._countdown = 0
        self._step = 0

    @property
    def events(self):
        return ['completed']

    def run(self):
        run = True
        while run:
            if self._step == len(self._method):
                run = False
            else:
                step = self._method[self._step]
            event, message, method = self._read_step(step)
            self.sockets.send('update', {'method': {'step':self._step}})
            method(event, message)
            self._step += 1
        self.sockets.send('update', {'method': {'complete':True}})

    def _read_step(self, step):
        event, message = parse_command(step, {})
        if message.get('units') in self._time_units and event == 'wait':
            method = self._wait_for_time
        elif event.startswith('wait'):
            method = self._wait
        elif event.startswith('goto'):
            message = {'value': int(step.replace('goto ', '')) - 1}
            method = self._goto
        else:
            method = self.sockets.send
        return event, message, method

    def _goto(self, event, message):
        self._step = message.get('value', 0)

    def _wait(self, event, message):
        """
        waits a 'completed' event
        
        """
        if event.startswith('wait for user'):
            step = event.replace('wait for user ', '')
            self.sockets.send('WAITFORUSER', {'event': step['event']})
        elif event.startswith('wait for'):
            step = event.replace('wait for ', '')
        self._do_wait(step)

    def _do_wait(self, step):
        event, message = self.sockets.recv() #only 'completed' events should be recieved
        completed = event.replace('completed ', '')
        if completed != step:
            self._do_wait(step)

    def _wait_for_time(self, event, message):
        seconds = None
        if message['units'] == 'minutes' or message['units'] == 'minute':
            seconds = message['value'] * 60.0
        elif message['units'] == 'hours' or message['units'] == 'hour':
            seconds = message['value'] * 60.0 * 60.0
        elif message['units'] == 'seconds' or message['units'] == 'second':
            seconds = message['value']
        if seconds:
            self._sleep(seconds)

    @property
    def countdown(self):
        return self._countdown

    def set_countdown(self, seconds):
        self._countdown = seconds
        self._sockets.send('update', {'method': {'countdown': int(self._countdown)}})

    def _sleep(self, seconds):
        countdown = Countdown(seconds, self.set_countdown)
        countdown.start()
        time.sleep(seconds)

def run_method(method, broadcast, addresses):
    runner = MethodRunner(method, broadcast, addresses)
    runner.start()
    



import abc

class IO(object):
    """
    All subclasses of Device have an 'io' property.  It should return
    an object that provides this interface.
    """
    
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def on(self):
        """
        turn on the io device
        """
        return

    @abc.abstractmethod
    def off(self):
        """
        turn the io device off
        """
        return

    @abc.abstractmethod
    def close(self):
        """
        close the io device
        """
        return

    @abc.abstractproperty
    def status(self):
        """
        status of the io device
        """
        return

    



import time, threading, random
from nose.tools import eq_
from gadgets import Gadgets, Addresses, Sockets, Broker
from gadgets.devices import ElectricHeater, electric_heater_factory
from gadgets.devices.heater.pwm import PWM

port = random.randint(3000, 50000)


class FakeGPIO(object):

    def __init__(self):
        self.addresses = Addresses(in_port=port, out_port=port+1)
        self.sockets = Sockets(self.addresses)
        self.status = False
        self.closed = False

    def on(self):
        self.sockets.send('test pwm on')
        self.status = True

    def off(self):
        self.sockets.send('test pwm off')
        self.status = False

    def close(self):
        self.sockets.close()
        self.closed = True
        
def get_fake_gpio():
    return FakeGPIO()

def get_pwm(*args, **kw):
    return PWM(get_fake_gpio)

    
class TestHeater(object):

    def setup(self):
        self.addresses = Addresses(in_port=port, out_port=port+1)
        self.sockets = Sockets(self.addresses, events=['test pwm'])
        
        self.heater = ElectricHeater(
            'living room',
            'heater',
            self.addresses,
            io_factory=get_pwm,
        )
        self.gadgets = Gadgets([self.heater], self.addresses)

    def teardown(self):
        self.sockets.send('shutdown')
        time.sleep(0.2)
        self.sockets.close()
        
    def test_create(self):
        pass

    def _read(self):
        with open(path, 'r') as f:
            return f.read()

    def test_on_and_off(self):
        t = threading.Thread(target=self.gadgets.start)
        t.start()
        time.sleep(1)
        self.sockets.send('heat living room', {'units': 'C', 'value': 88})
        event, message = self.sockets.recv()
        eq_(event, 'test pwm on')
        eq_(self.heater.io.duty_percent, 100)
        self.sockets.send('update temperature', {'living room': {'temperature': {'value': 87, 'units': 'C'}}})
        event, message = self.sockets.recv()
        eq_(event, 'test pwm on')
        eq_(self.heater.io.duty_percent, 25)
        self.sockets.send('update temperature', {'living room': {'temperature': {'value': 88, 'units': 'C'}}})
        event, message = self.sockets.recv()
        eq_(event, 'test pwm on')
        eq_(self.heater.io.duty_percent, 0)
        time.sleep(1)



import re, json, os, copy, time, curses
from os import system
import curses


class Container(object):

    _sort = lambda a, b : a if a == 'temperature' else b
    
    def __init__(self, stdscr, row, col, data, name, command_mode):
        self._name = name
        self.row = row
        self._col = col
        self._win = stdscr
        self._data = data
        self._command_mode = command_mode
        self._on_str = lambda x: x and 'On' or 'Off'

    def _get_state(self):
        state = dict(devices={})
        for name, value in self._data.iteritems():
            state['devices'][name] = value
        return state

    def draw(self):
        color = self._get_color(self._name, 0)
        self._win.addstr(self.row, self._col, self._name, color)
        self.row += 1
        state = self._get_state()
        keys = state['devices'].keys()
        if 'temperature' in keys:
            value = state['devices'].pop('temperature')
            self._write_data('temperature', value)
        if 'volume' in keys:
            value = state['devices'].pop('volume')
            self._write_data('volume', value)
        for key, value in state['devices'].iteritems():
            self._write_on_off(key, value)
        return self.row

    def _get_color(self, name, i):
        color = 0
        if self._command_mode.is_active:
            keys = self._command_mode.keys
            if len(keys) > i and keys[i] == name and keys[0] == self._name:
                color = curses.A_BOLD
        return color

    def _write_data(self, label, val):
        val = val['value']
        val = '{0}{1:>{2}.2f}'.format(label, val, 20 - len(label))
        self._win.addstr(self.row, self._col + 4, val)
        self.row += 1

    def _write_on_off(self, label, val):
        #val, target = val
        if val is None:
            return
        if val['value'] is not True and val['value'] is not False:
            s = val['value']
        else:
            s = self._on_str(val['value'])
        target = val.get('target')
        if isinstance(target, float):
            s = '{0} ({1:.2f})'.format(s, target)
        color = self._get_color(label, 1)
        self._win.addstr(self.row, self._col + 4, label, color)
        if val['value'] is True:
            self._win.addstr(self.row, self._col + 24 - len(s), s, curses.A_BOLD | curses.color_pair(2))
        else:
            self._win.addstr(self.row, self._col + 24 - len(s), s, curses.A_BOLD | curses.color_pair(3))
        self.row += 1
        



from mmap import mmap
import struct

def main():
    """
    Borrowed from Dan Watts at
    http://www.gigamegablog.com/docs/setPWMReg.py
    """
    MMAP_OFFSET = 0x44c00000                # base address of registers
    MMAP_SIZE   = 0x48ffffff-MMAP_OFFSET    # size of the register memory space
    CM_PER_BASE = 0x44e00000 - MMAP_OFFSET
    CM_PER_EPWMSS1_CLKCTRL = CM_PER_BASE + 0xcc
    CM_PER_EPWMSS0_CLKCTRL = CM_PER_BASE + 0xd4
    CM_PER_EPWMSS2_CLKCTRL = CM_PER_BASE + 0xd8
    with open('/tmp/pwm', 'w') as f:
        f.write('pwm enabled')
    with open("/dev/mem", "r+b") as f:
        mem = mmap(f.fileno(), MMAP_SIZE, offset=MMAP_OFFSET)
    def _andReg(address, mask):
        """ Sets 32-bit Register at address to its current value AND mask. """
        _setReg(address, _getReg(address)&mask)
    def _orReg(address, mask):
        """ Sets 32-bit Register at address to its current value OR mask. """
        _setReg(address, _getReg(address)|mask)
    def _xorReg(address, mask):
        """ Sets 32-bit Register at address to its current value XOR mask. """
        _setReg(address, _getReg(address)^mask)
    def _getReg(address):
        """ Returns unpacked 32 bit register value starting from address. """
        return struct.unpack("<L", mem[address:address+4])[0]
    def _setReg(address, new_value):
        """ Sets 32 bits at given address to given value. """
        mem[address:address+4] = struct.pack("<L", new_value)
    val = _getReg(CM_PER_EPWMSS1_CLKCTRL)
    print "Register CM_PER_EPWMSS1_CLKCTRL was " + hex(val)
    _setReg(CM_PER_EPWMSS1_CLKCTRL, 0x2)
    val = _getReg(CM_PER_EPWMSS1_CLKCTRL)
    print "Register CM_PER_EPWMSS1_CLKCTRL changed to " + hex(val)

if __name__ == '__main__':
    main()



from gadgets.devices.device import Device
from gadgets.devices.heater.triggers.temperature import TemperatureTriggerFactory

class ElectricHeater(Device):
    """
    Used to control electric heating coils on my brewery.  Here
    is an example of its use:

    >>> from gadgets import get_gadgets
    >>> from gadgets.pins.beaglebone import pins
    >>> brewery_def = {
    ...     'locations': {
    ...         'boiler': {
    ...             'heater': {
    ...                 'type': 'electric heater',
    ...                 'pin': pins['pwm'][8][13],
    ...             }
    ...         }
    ...     }
    ... }
    >>> brewery = get_gadgets(brewery_def)
    >>> brewery.start()
    
    If you turn on the electric heater with a target temperature
    like this:

    >>> from gadgets import Sockets
    >>> sock = Sockets()
    >>> sock.send('heat boiler 180 F')
    
    then as the temperature of the pot approaches the target temperature
    it will use its pwm pin to apply less heat to the pot so the heater
    won't over heat the container.
    """

    _units = ['celcius', 'C', 'fahrenheit', 'F']
    _on_template = 'heat {location}'
    _off_template = 'stop heating {location}'

    def __init__(self, location, name, addresses, io_factory=None, trigger_factory=None, on=None, off=None):
        super(ElectricHeater, self).__init__(location, name, addresses, io_factory, trigger_factory, on, off)
        self._target_temperature = None
        self._trigger_factory = TemperatureTriggerFactory(location, addresses)

    @property
    def events(self):
        return super(ElectricHeater, self).events + ['update temperature']

    def _get_trigger(self, message):
        return self._trigger_factory(self._on_event, self._off_event, message, target=None)

    def event_received(self, event, message):
        if event.startswith('update temperature'):
            self._update_pwm(message)
        else:
            super(ElectricHeater, self).event_received(event, message)

    def on(self, message):
        self._target_temperature = self._get_temperature(message)
        super(ElectricHeater, self).on(message)

    def off(self, message=None):
        self._target_temperature = None
        super(ElectricHeater, self).off(message)

    def _get_temperature(self, message):
        if message.get('units') in self._units:
            units = message['units']
            value = message['value']
            if units == 'F' or units == 'fahrenheit':
                value = (value * 1.8) + 32.0
            return value

    def _update_pwm(self, message):
        if not self.io.status:
            return
        if self._location in message:
            temperature = self._get_temperature(message[self._location]['temperature'])
            self.io.duty_percent = self._get_duty_percent(temperature)

    def _get_duty_percent(self, temperature):
        if self._target_temperature is None:
            duty_percent = 100
        else:
            difference = self._target_temperature - temperature
            if difference is None or difference <= 0:
                duty_percent = 0
            elif difference <= 1:
                duty_percent = 25
            elif difference <= 2:
                duty_percent = 50
            else:
                duty_percent = 100
        return duty_percent


        
        



import curses, time
from gadgets.ui.window import Window

class Help(Window):
    """get help"""

    def __init__(self, lock, dialogs, db):
        self._dialogs = dialogs
        self._lock = lock
        self._width = 60
        self._height = 18

    def __call__(self, screen, data):
        self._lock.acquire()
        win = self._get_subwin(screen)
        self._add_title(win, 'help')
        row = 2
        for key, value in self._dialogs.iteritems():
            doc = value.callable.__doc__
            if doc is not None:
                win.addstr(row, 2, '{0}:  {1}'.format(chr(key), doc))
                row += 1
        row += 1
        win.addstr(row, 2, 'hit any key to exit help')
        win.move(row+1, 2)
        x = win.getch()
        self._lock.release()